@@ -3,15 +3,16 @@
 #include "Singular/tok.h"
 #include "Singular/subexpr.h"
 #include "Singular/ipshell.h"
+#include "Singular/ipid.h"
 
 #include "Singular/blackbox.h"
 
 #define MAX_BB_TYPES 256
 // #define BLACKBOX_DEVEL 1
 
-static blackbox* blackboxTable[MAX_BB_TYPES];
-static char *    blackboxName[MAX_BB_TYPES];
-static int blackboxTableCnt=0;
+GLOBAL_VAR static blackbox* blackboxTable[MAX_BB_TYPES];
+GLOBAL_VAR static char *    blackboxName[MAX_BB_TYPES];
+GLOBAL_VAR static int blackboxTableCnt=0;
 #define BLACKBOX_OFFSET (MAX_TOK+1)
 blackbox* getBlackboxStuff(const int t)
 {
@@ -36,6 +37,21 @@ void *blackbox_default_Copy(blackbox */*b*/,void */*d*/)
   WerrorS("missing blackbox_Copy");
   return NULL;
 }
+BOOLEAN blackbox_default_Assign(leftv l, leftv r)
+{
+  int lt=l->Typ();
+  blackbox* b=getBlackboxStuff(lt);
+  if ((lt==r->Typ())
+  && (l->Data()!=r->Data()))
+  {
+    b->blackbox_destroy(b,(void*)l->Data());
+    if (l->rtyp==IDHDL)
+      IDDATA((idhdl)l->data)=(char*)b->blackbox_Copy(b,r->Data());
+    else
+      l->data=b->blackbox_Copy(b,r->Data());
+  }
+  return FALSE;
+}
 void blackbox_default_Print(blackbox *b,void *d)
 {
   char *s=b->blackbox_String(b,d);
@@ -125,19 +141,28 @@ BOOLEAN blackbox_default_Check(blackbox *,leftv,leftv)
 }
 int setBlackboxStuff(blackbox *bb, const char *n)
 {
-  int where=-1;
-  if (MAX_BB_TYPES<=blackboxTableCnt)
+  int where = -1;
+  for (int i=0;i<MAX_BB_TYPES;i++)
   {
-    // second try, find empty slot from removed bb:
-    for (int i=0;i<MAX_BB_TYPES;i++)
-    {
-      if (blackboxTable[i]==NULL) { where=i; break; }
+    if (blackboxTable[i]!=NULL && strcmp(blackboxName[i],n)==0) {
+      where = i;
+      break;
     }
   }
-  else
-  {
-    where=blackboxTableCnt;
-    blackboxTableCnt++;
+  if (where < 0) {
+    if (MAX_BB_TYPES<=blackboxTableCnt)
+    {
+      // second try, find empty slot from removed bb:
+      for (int i=0;i<MAX_BB_TYPES;i++)
+      {
+        if (blackboxTable[i]==NULL) { where=i; break; }
+      }
+    }
+    else
+    {
+      where=blackboxTableCnt;
+      blackboxTableCnt++;
+    }
   }
   if (where==-1)
   {
@@ -165,6 +190,7 @@ int setBlackboxStuff(blackbox *bb, const char *n)
     if (bb->blackbox_Print==NULL)   bb->blackbox_Print=blackbox_default_Print;
     if (bb->blackbox_Init==NULL)    bb->blackbox_Init=blackbox_default_Init;
     if (bb->blackbox_Copy==NULL)    bb->blackbox_Copy=blackbox_default_Copy;
+    if (bb->blackbox_Assign==NULL)  bb->blackbox_Assign=blackbox_default_Assign;
     if (bb->blackbox_Op1==NULL)     bb->blackbox_Op1=blackboxDefaultOp1;
     if (bb->blackbox_Op2==NULL)     bb->blackbox_Op2=blackboxDefaultOp2;
     if (bb->blackbox_Op3==NULL)     bb->blackbox_Op3=blackboxDefaultOp3;
@@ -214,3 +240,23 @@ void printBlackboxTypes()
        Print("type %d: %s\n",i,blackboxName[i]);
   }
 }
+
+struct blackbox_list *getBlackboxTypes()
+{
+	int i = 0;
+	void **l = (void **)omalloc0(blackboxTableCnt * sizeof(void *));
+	struct blackbox_list *list_struct = (struct blackbox_list *) omAlloc0(sizeof(struct blackbox_list));
+	list_struct->count = blackboxTableCnt;
+	list_struct->list = l;
+
+	for (i = blackboxTableCnt-1; i >= 0 ;i--)
+	{
+		if (blackboxName[i]!=NULL) {
+			l[i] = (void *)omStrDup(blackboxName[i]);
+			//Print("type %d: %s\n",i,blackboxName[i]);
+		} else {
+			l[i] = NULL;
+		}
+	}
+	return list_struct;
+}
