@@ -1,5 +1,4 @@
 #include "kernel/mod2.h"
-#ifdef HAVE_PYTHON_MOD
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -7,20 +6,19 @@
 
 #include <unistd.h>
 #include <sys/stat.h>
-#include <boost/python.hpp>
-#include <Python.h>
 #include "Singular/tok.h"
 #include "kernel/structs.h"
 #include "Singular/mod_lib.h"
 #include "Singular/ipid.h"
 
 #include "Singular/locals.h"
+
 #include "omalloc/omalloc.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <time.h>
+#ifdef HAVE_PYTHON
+#include <boost/python.hpp>
+#include <Python.h>
 #include <Python.h>
 #include "wrapper.h"
 
@@ -58,8 +56,8 @@ static BOOLEAN mod_python(leftv __res, leftv __h)
 //  )
 extern "C" int SI_MOD_INIT(python_module)(SModulFunctions* psModulFunctions)
 {
-  char *py=getenv("PYTHONPATH");
-  char buf[1024];
+  VAR char *py=getenv("PYTHONPATH");
+  VAR char buf[1024];
   if (py!=NULL)
   {
     strcpy(buf,py);
@@ -80,4 +78,10 @@ path.insert(0,'.')\n");
   psModulFunctions->iiAddCproc(currPack->libname,"python",FALSE, mod_python);
   return MAX_TOK;
 }
+#else
+extern "C" int SI_MOD_INIT(python_module)(SModulFunctions* psModulFunctions)
+{
+  PrintS("no python_mod support\n");
+  return MAX_TOK;
+}
 #endif
