@@ -6,7 +6,7 @@
 */
 
 
-
+#include <string.h>
 
 #include "kernel/mod2.h"
 
@@ -22,6 +22,9 @@
 
 #include "fehelp.h"
 
+#ifdef HAVE_FLINT
+#include <flint/flint.h>
+#endif
 
 const char SHORT_OPTS_STRING[] = "bdhpqstvxec:r:u:";
 
@@ -134,6 +137,7 @@ feOptIndex feGetOptIndex(int optc)
 // Return: NULL -- everything ok
 //         "error-string" on error
 #if !defined(ESINGULAR) && !defined(TSINGULAR)
+
 #include "omalloc/omalloc.h"
 #include "resources/feResource.h"
 #include "kernel/oswrapper/feread.h"
@@ -307,6 +311,23 @@ static const char* feOptAction(feOptIndex opt)
         return NULL;
       }
 
+      #ifdef HAVE_FLINT
+      #if __FLINT_RELEASE >= 20503
+      case FE_OPT_FLINT_THREADS:
+      {
+        slong nthreads = (slong)feOptSpec[FE_OPT_FLINT_THREADS].value;
+        nthreads = FLINT_MAX(nthreads, WORD(1));
+        flint_set_num_threads(nthreads);
+        int * cpu_affinities = new int[nthreads];
+        for (slong i = 0; i < nthreads; i++)
+          cpu_affinities[i] = (int)i;
+        flint_set_thread_affinity(cpu_affinities, nthreads);
+        delete[] cpu_affinities;
+        return NULL;
+      }
+      #endif
+      #endif
+
       default:
         return NULL;
   }
@@ -351,7 +372,7 @@ void fePrintOptValues()
 void feOptHelp(const char* name)
 {
   int i = 0;
-  char tmp[20];
+  char tmp[60];
 #if defined(ESINGULAR)
   printf("ESingular starts up Singular within emacs;\n");
 #elif defined(TSINGULAR)
@@ -395,7 +416,7 @@ void feOptHelp(const char* name)
   }
 
   printf("\nFor more information, type `help;' from within Singular or visit\n");
-  printf("http://www.singular.uni-kl.de or consult the\n");
+  printf("https://www.singular.uni-kl.de or consult the\n");
   printf("Singular manual (available as on-line info or html manual).\n");
 }
 
