@@ -30,6 +30,8 @@
 
 VAR char fe_promptstr[] ="  ";
 VAR FILE *File_Profiling=NULL;
+VAR FILE *File_Log=NULL;
+VAR BOOLEAN File_Log_written=FALSE;
 
 // line buffer for reading:
 // minimal value for MAX_FILE_BUFFER: 4*4096 - see Tst/Long/gcd0_l.tst
@@ -547,8 +549,8 @@ int feReadLine(char* b, int l)
           else          len=ss-anf;
           show_echo=TRUE;
         }
-        else if (/*(startfptr>0) &&*/
-        (currentVoice->buffer[startfptr-1]=='\n'))
+        else if /*(startfptr>0) &&*/
+        (currentVoice->buffer[startfptr-1]=='\n')
         {
           anf=currentVoice->buffer+startfptr;
           const char *ss=strchr(anf,'\n');
@@ -616,6 +618,11 @@ int feReadLine(char* b, int l)
     {
       fputs(s,feProtFile);
     }
+    if (File_Log!=NULL)
+    {
+      File_Log_written=TRUE;
+      fputs(s,File_Log);
+    }
     int rc=fePrintEcho(s,b)+1;
     //s[strlen(s)+1]='\0'; add an second \0 at the end of the string
     s[rc]='\0';
