@@ -314,7 +314,10 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
             Warn("redefining %s (%s) %s:%d",s,my_yylinebuf,f, yylineno);
         }
         if (s==IDID(h)) IDID(h)=NULL;
-        killhdl2(h,root,currRing);
+        if((t!=PROC_CMD)||(IDPROC(h)->language!=LANG_C))
+        {
+          killhdl2(h,root,currRing);
+        }
       }
     }
     else
@@ -336,6 +339,7 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
             Warn("redefining %s (%s) %s:%d",s,my_yylinebuf,f, yylineno);
         }
         if (s==IDID(h)) IDID(h)=NULL;
+        //  proc is not ring-dep, no need to check for type "proc":
         killhdl2(h,&currRing->idroot,currRing);
       }
       else
@@ -358,7 +362,10 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
             Warn("redefining %s (%s) %s:%d",s,my_yylinebuf,f, yylineno);
         }
         if (s==IDID(h)) IDID(h)=NULL;
-        killhdl2(h,&IDROOT,NULL);
+        if((t!=PROC_CMD)||(IDPROC(h)->language!=LANG_C))
+        {
+          killhdl2(h,&IDROOT,NULL);
+        }
       }
       else
         goto errlabel;
@@ -374,7 +381,7 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
     //Werror("identifier `%s` in use(lev h=%d,typ=%d,t=%d, curr=%d)",s,IDLEV(h),IDTYP(h),t,lev);
     Werror("identifier `%s` in use",s);
     //listall();
-    omFree((ADDRESS)s);
+    omFreeBinAddr((ADDRESS)s);
     return NULL;
 }
 void killid(const char * id, idhdl * ih)
@@ -458,7 +465,8 @@ void killhdl2(idhdl h, idhdl * ih, ring r)
   }
   if (IDTYP(h) == PACKAGE_CMD)
   {
-    if (((IDPACKAGE(h)->language==LANG_C)&&(IDPACKAGE(h)->idroot!=NULL))
+    if ((((IDPACKAGE(h)->language==LANG_C) ||(IDPACKAGE(h)->language==LANG_MIX))
+      &&(IDPACKAGE(h)->idroot!=NULL))
     || (strcmp(IDID(h),"Top")==0))
     {
       Warn("cannot kill `%s`",IDID(h));
@@ -482,7 +490,7 @@ void killhdl2(idhdl h, idhdl * ih, ring r)
         hdh = temp;
       }
       killhdl2(*hd,hd,NULL);
-      if (IDPACKAGE(h)->libname!=NULL) omFree((ADDRESS)(IDPACKAGE(h)->libname));
+      if (IDPACKAGE(h)->libname!=NULL) omFreeBinAddr((ADDRESS)(IDPACKAGE(h)->libname));
     }
     paKill(IDPACKAGE(h));
     if (currPackHdl==h) currPackHdl=packFindHdl(currPack);
@@ -492,10 +500,11 @@ void killhdl2(idhdl h, idhdl * ih, ring r)
     rKill(h);
   else if (IDDATA(h)!=NULL)
     s_internalDelete(IDTYP(h),IDDATA(h),r);
+  IDDATA(h)=NULL;
   //  general  -------------------------------------------------------------
   // now dechain it and delete idrec
   if (IDID(h)!=NULL) // OB: ?????
-    omFree((ADDRESS)IDID(h));
+    omFreeBinAddr((ADDRESS)IDID(h));
   IDID(h)=NULL;
   IDDATA(h)=NULL;
   if (h == (*ih))
@@ -754,9 +763,9 @@ BOOLEAN piKill(procinfov pi)
       }
     }
     if (pi->libname != NULL) // OB: ????
-      omFree((ADDRESS)pi->libname);
+      omFreeBinAddr((ADDRESS)pi->libname);
     if (pi->procname != NULL) // OB: ????
-      omFree((ADDRESS)pi->procname);
+      omFreeBinAddr((ADDRESS)pi->procname);
 
     if( pi->language == LANG_SINGULAR)
     {
@@ -785,7 +794,7 @@ void paCleanUp(package pack)
       dynl_close (pack->handle);
 #endif /* HAVE_DYNAMIC_LOADING */
     }
-    omFree((ADDRESS)pack->libname);
+    omFreeBinAddr((ADDRESS)pack->libname);
     memset((void *) pack, 0, sizeof(sip_package));
     pack->language=LANG_NONE;
   }
@@ -877,8 +886,8 @@ BOOLEAN iiAlias(leftv p)
       case MAP_CMD:
          {
            map im = IDMAP(pp);
-           omFree((ADDRESS)im->preimage); 
-	   im->preimage=NULL;// and continue
+           omFreeBinAddr((ADDRESS)im->preimage);
+           im->preimage=NULL;// and continue
          }
          // continue as ideal:
       case IDEAL_CMD:
