@@ -7,14 +7,15 @@
 /*
 * ABSTRACT: identfier handling
 */
+#include "misc/options.h"
 #include "Singular/idrec.h"
 #include "Singular/subexpr.h"
 #include "Singular/lists.h"
 
-extern idhdl currPackHdl;
-extern idhdl basePackHdl;
-extern package currPack;
-extern package basePack;
+EXTERN_VAR idhdl currPackHdl;
+EXTERN_VAR idhdl basePackHdl;
+EXTERN_VAR package currPack;
+EXTERN_VAR package basePack;
 #define IDROOT (currPack->idroot)
 
 struct sip_command;
@@ -62,7 +63,7 @@ class proclevel
   void    push(char *);
   void    pop();
 };
-extern proclevel *procstack;
+EXTERN_VAR proclevel *procstack;
 
 typedef struct
 {
@@ -74,7 +75,7 @@ typedef struct
 } SModulFunctions;
 
 
-extern idhdl      currRingHdl;
+EXTERN_VAR idhdl      currRingHdl;
 /* ================================================================== */
 /* module support */
 typedef int (*SModulFunc_t)(SModulFunctions*);
@@ -86,7 +87,7 @@ void module_help_proc(const char *newlib,const char *p, const char *help);
 
 /*extern ring     currRing;  in ring.h */
 
-idhdl enterid(const char * a, int lev, int t, idhdl* root, BOOLEAN init=TRUE, BOOLEAN serach=TRUE);
+idhdl enterid(const char * a, int lev, int t, idhdl* root, BOOLEAN init=TRUE, BOOLEAN search=TRUE);
 idhdl ggetid(const char *n);
 //idhdl ggetid(const char *n, idhdl *packhdl);
 void  killid(const char * a, idhdl * i);
@@ -97,13 +98,17 @@ lists ipNameListLev(idhdl root, int lev);
 void  ipMoveId(idhdl h);
 BOOLEAN checkPackage(package pack);
 idhdl packFindHdl(package r);
-void jjNormalizeQRingP(poly &p);
+poly jj_NormalizeQRingP(poly p, const ring r);
 void jjNormalizeQRingId(leftv I);
+#define jjNormalizeQRingP(p) jj_NormalizeQRingP(p,currRing)
 void *idrecDataInit(int t);
 
-#define FLAG_STD   0
-#define FLAG_TWOSTD  3
-#define FLAG_QRING   4
+#define FLAG_STD        0 // is a SB wrt. currRing ("isSB")
+#define FLAG_TWOSTD     3 // is a 2-sided SB wrt. currRing (" 2SB")
+#define FLAG_QRING      4 // is normalized wrt. qring ("qringNF")
+#define FLAG_QRING_DEF  5 // is a qring to be defined
+#define FLAG_OTHER_RING 6 // mark newstruct parts as "write-only"
+#define FLAG_RING       7 // mark newstruct parts as "ring dep."
 #define hasFlag(A,F) Sy_inset((F),(A)->flag)
 #define setFlag(A,F) (A)->flag|=Sy_bit(F)
 #define resetFlag(A,F) (A)->flag&=~Sy_bit(F)
@@ -134,12 +139,12 @@ void ipListFlag(idhdl h);
 #define IDPACKAGE(a) ((a)->data.pack)
 #define IDPROC(a)   ((a)->data.pinf)
 
-extern omBin sip_command_bin;
-extern omBin sip_package_bin;
-extern omBin idrec_bin;
-extern omBin sleftv_bin;
+EXTERN_VAR omBin sip_command_bin;
+EXTERN_VAR omBin sip_package_bin;
+EXTERN_VAR omBin idrec_bin;
+EXTERN_VAR omBin sleftv_bin;
 
-extern coeffs coeffs_BIGINT;
+EXTERN_VAR coeffs coeffs_BIGINT;
 
 /* options */
 struct soptionStruct
