@@ -14,7 +14,6 @@
 
 #ifdef HAVE_DBM
 
-#  include "omalloc/omalloc.h"
 #  include "Singular/tok.h"
 #  include "Singular/ipid.h"
 #  include "Singular/links/silink.h"
@@ -83,7 +82,7 @@ LINKAGE BOOLEAN dbOpen(si_link l, short flag, leftv u)
     else
       SI_LINK_SET_R_OPEN_P(l);
     l->data=(void *)(db);
-    omFree(l->mode);
+    omFreeBinAddr(l->mode);
     l->mode=omStrDup(mode);
     return FALSE;
   }
@@ -104,7 +103,7 @@ LINKAGE BOOLEAN dbClose(si_link l)
 }
 
 //**************************************************************************/
-static datum d_value;
+STATIC_VAR datum d_value;
 LINKAGE leftv dbRead2(si_link l, leftv key)
 {
   GDBM_info *db = (GDBM_info *)l->data;
@@ -203,7 +202,7 @@ LINKAGE BOOLEAN dbWrite(si_link l, leftv key)
 //   db = (GDBM_info *)omAlloc0(sizeof *db);
 //   db = (GDBM_info *)l->data;
   BOOLEAN b=TRUE;
-  register int ret;
+  REGISTER int ret;
 
   if (strcmp(l->mode,"rw")!=0) // r-mode
     {
@@ -309,7 +308,7 @@ LINKAGE BOOLEAN dbOpen(si_link l, short flag, leftv /*u*/)
     else
       SI_LINK_SET_R_OPEN_P(l);
     l->data=(void *)(db);
-    omFree(l->mode);
+    omFreeBinAddr(l->mode);
     l->mode=omStrDup(mode);
     return FALSE;
   }
@@ -329,7 +328,7 @@ LINKAGE BOOLEAN dbClose(si_link l)
 }
 
 //**************************************************************************/
-static datum d_value;
+STATIC_VAR datum d_value;
 LINKAGE leftv dbRead2(si_link l, leftv key)
 {
   DBM_info *db = (DBM_info *)l->data;
@@ -385,7 +384,7 @@ LINKAGE BOOLEAN dbWrite(si_link l, leftv key)
 {
   DBM_info *db = (DBM_info *)l->data;
   BOOLEAN b=TRUE;
-  register int ret;
+  int ret;
 
   // database is opened
   if((key!=NULL) && (key->Typ()==STRING_CMD) )
