@@ -38,7 +38,7 @@
 #include "feOpt.h"
 #include "links/silink.h"
 #include "mod_lib.h"
-#include "Singular/distrib.h"
+#include "misc/distrib.h"
 
 #include "misc/options.h"
 #include "misc/intvec.h"
@@ -695,7 +695,7 @@ BOOLEAN setOption(leftv res, leftv v)
   okay:
     if (currRing != NULL)
       currRing->options = si_opt_1 & TEST_RINGDEP_OPTS;
-    omFree((ADDRESS)n);
+    omFreeBinAddr((ADDRESS)n);
     v=v->next;
   } while (v!=NULL);
 
@@ -764,7 +764,7 @@ extern "C"
 #endif
 
 #ifndef MAKE_DISTRIBUTION
-const char *singular_date=__DATE__ " " __TIME__;
+const char *singular_date = __DATE__ " " __TIME__;
 #endif
 
 char * versionString(/*const bool bShowDetails = false*/ )
@@ -1099,7 +1099,21 @@ void m2_end(int i)
   if (!m2_end_called)
   {
     EXTERN_VAR FILE* File_Profiling;
+    EXTERN_VAR FILE* File_Log;
+    EXTERN_VAR BOOLEAN File_Log_written;
     if (File_Profiling!=NULL) { fclose(File_Profiling); File_Profiling=NULL; }
+    if (File_Log!=NULL)
+    {
+      fclose(File_Log);
+      File_Log=NULL;
+      if (File_Log_written==FALSE) // remove empty logs
+      {
+        int pid=getpid();
+        char buf[20];
+        sprintf(buf,"/tmp/sing_log.%d",pid);
+        remove(buf);
+      }
+    }
     m2_end_called = TRUE;
 #ifdef HAVE_SIMPLEIPC
     for (int j = SIPC_MAX_SEMAPHORES-1; j >= 0; j--)
@@ -1118,7 +1132,6 @@ void m2_end(int i)
       }
     }
 #endif   // HAVE_SIMPLEIPC
-    fe_reset_input_mode();
     monitor(NULL,0);
 #ifdef PAGE_TEST
     mmEndStat();
