@@ -21,7 +21,7 @@
 #include "ipid.h"
 #include "sdb.h"
 #include "feOpt.h"
-#include "distrib.h"
+#include "misc/distrib.h"
 #include "mmalloc.h"
 #include "tok.h"
 #include "resources/fegetopt.h"
@@ -35,10 +35,12 @@
 #endif
 
 
-extern int siInit(char *);
+extern void siInit(char *);
 #ifdef PSINGULAR
 GLOBAL_VAR char *global_argv0;
 #endif
+EXTERN_VAR FILE* File_Log;
+EXTERN_VAR BOOLEAN FE_OPT_NO_SHELL_FLAG;
 
 int mmInit( void )
 {
@@ -139,17 +141,29 @@ int main(          /* main entry to Singular */
 " by: W. Decker, G.-M. Greuel, G. Pfister, H. Schoenemann     \\   %s\n"
 "FB Mathematik der Universitaet, D-67653 Kaiserslautern        \\\n"
 , VERSION, VERSION_DATE);
-  if (feOptValue(FE_OPT_NO_SHELL)) WarnS("running in restricted mode:"
-    " shell invocation and links are disallowed");
+    if (feOptValue(FE_OPT_NO_SHELL))
+    {
+      WarnS("running in restricted mode:"
+      " shell invocation and links are disallowed");
+      FE_OPT_NO_SHELL_FLAG=TRUE;
+    }
   }
   else
   {
+    FE_OPT_NO_SHELL_FLAG=FALSE;
     if (feOptValue(FE_OPT_SORT)) On(SW_USE_NTL_SORT);
     dup2(1,2);
     /* alternative:
     *    memcpy(stderr,stdout,sizeof(FILE));
     */
   }
+  if (feOptValue(FE_OPT_LOG)!=NULL)
+  {
+    int pid=getpid();
+    char *buf=(char*)malloc(10+strlen((char*) feOptSpec[FE_OPT_LOG].value));
+    sprintf(buf,"%s.%d",(char*) feOptSpec[FE_OPT_LOG].value,pid);
+    File_Log=fopen(buf,"w");
+  }
 
 #ifdef SINGULAR_PYOBJECT_SETUP_H
    pyobject_setup();
