@@ -14,8 +14,12 @@
 #include "config.h"
 
 
-#include "cf_switches.h"
 #include "cf_defs.h"
+#include "cf_switches.h"
+
+#ifdef HAVE_FLINT
+#include <flint/flint.h>
+#endif
 
 /** CFSwitches::CFSwitches ()
  *
@@ -29,14 +33,23 @@ CFSwitches::CFSwitches ()
     for ( int i = 0; i < CFSwitchesMax; i++ )
         switches[i] = false;
 // and set the default (recommended) On-values:
-#ifdef HAVE_NTL
+#if defined(HAVE_NTL) || defined(HAVE_FLINT)
   On(SW_USE_CHINREM_GCD);
-  //Off(SW_USE_NTL_SORT);
+#endif
+  On(SW_USE_NTL_SORT);
+#ifdef HAVE_FLINT
+  On(SW_USE_FL_GCD_P);
+  On(SW_USE_FL_GCD_0);
+#if (__FLINT_RELEASE >= 20700)  
+  On(SW_USE_FL_FAC_P);
+#endif
 #endif
   On(SW_USE_EZGCD);
-  //On(SW_USE_EZGCD_P); // still testing
+  On(SW_USE_EZGCD_P);
   On(SW_USE_QGCD);
+  On(SW_USE_FL_FAC_0);
+  //On(SW_USE_FL_FAC_0A);
 }
 
-CFSwitches cf_glob_switches;
+INST_VAR CFSwitches cf_glob_switches;
 
