@@ -11,7 +11,6 @@
 #include <omp.h>
 #include "kernel/structs.h"
 #include "kernel/GBEngine/kutil.h"
-#include "omalloc/omalloc.h"
 #include "kernel/polys.h"
 #include "polys/monomials/p_polys.h"
 #include "polys/templates/p_Procs.h"
@@ -29,20 +28,20 @@
 
 #define pDeg(A) p_Deg(A,currRing)
 
-int notInG              =   0;
-int numberOfRules       =   0;
-int reductionsToZero    =   0;
-int reductionTime       =   0;
-int spolsTime           =   0;
-int highestDegree       =   0;
-int degreeBound         =   0;
-int numberUsefulPairs   =   0;
-int numberUselessPairs  =   0;
-int numberUsefulPairsMinDeg = 0;
-int highestDegreeGBCriticalPair = 0;
-int numberRejectedF5CriticalPairs = 0;
-int numberOfReductions  = 0;
-int numberOfSpolys  = 0;
+VAR int notInG              =   0;
+VAR int numberOfRules       =   0;
+VAR int reductionsToZero    =   0;
+VAR int reductionTime       =   0;
+VAR int spolsTime           =   0;
+VAR int highestDegree       =   0;
+VAR int degreeBound         =   0;
+VAR int numberUsefulPairs   =   0;
+VAR int numberUselessPairs  =   0;
+VAR int numberUsefulPairsMinDeg = 0;
+VAR int highestDegreeGBCriticalPair = 0;
+VAR int numberRejectedF5CriticalPairs = 0;
+VAR int numberOfReductions  = 0;
+VAR int numberOfSpolys  = 0;
 /*
 ====================================================================
 sorting ideals by decreasing total degree "left" and "right" are the
@@ -152,12 +151,12 @@ LList* F5inc(int i, poly f_i, LList* gPrev, LList* reducers, ideal gbPrev, poly
     // computation of critical pairs with checking of criterion 1 and criterion 2 and saving them
     // in the list critPairs
     criticalPair(gPrev, critPairs, lTag, rTag, rules, rejectedGBList, plus);
-    static LList* sPolyList        =   new LList();
+    STATIC_VAR LList* sPolyList        =   new LList();
     //sPolyList->print();
     // labeled polynomials which have passed reduction() and have to be added to list gPrev
-    static LList* completed        =   new LList();
+    STATIC_VAR LList* completed        =   new LList();
     // the reduced labeled polynomials which are returned from subalgorithm reduction()
-    static LList* reducedLPolys     =   new LList();
+    STATIC_VAR LList* reducedLPolys     =   new LList();
     // while there are critical pairs to be further checked and deleted/computed
     while(NULL != critPairs->getFirst()) {
         // critPairs->getMinDeg() deletes the first elements of minimal degree from
