@@ -45,7 +45,7 @@ void khCheck( ideal Q, intvec *w, intvec *hilb, int &eledeg, int &count,
 * The weights w are needed in the module case, otherwise NULL.
 */
 {
-  intvec *newhilb;
+  intvec *newhilb,*new1;
   int deg,l,ln,mw;
   pFDegProc degp;
 
@@ -77,7 +77,7 @@ void khCheck( ideal Q, intvec *w, intvec *hilb, int &eledeg, int &count,
     // degp = pWDegree;
     l = hilb->length()-1;
     mw = (*hilb)[l];
-    newhilb = hHstdSeries(strat->Shdl,w,strat->kHomW,Q,strat->tailRing);
+    newhilb =hFirstSeries(strat->Shdl,w,Q,strat->kHomW);
     ln = newhilb->length()-1;
     deg = degp(strat->P.p,currRing)-mw;
     loop // compare the series in degree deg, try to increase deg -----------
@@ -134,20 +134,20 @@ void khCheckLocInhom(ideal Q, intvec *w, intvec *hilb, int &count,
              kStrategy strat)
 
 /*
-This will be used for the local orderings in the case of the inhomogenous ideals.
+This will be used for the local orderings in the case of the inhomogeneous ideals.
 Assume f1,...,fs are already in the standard basis. Test if hilb(LM(f1),...,LM(fs),1)
-is equal to the inputed one.
+is equal to the inputted one.
 If no, do nothing.
 If Yes, we know that all polys that we need are already in the standard basis
 so delete all the remaining pairs
 */
 {
   ideal Lm;
-  intvec *newhilb;
+  intvec *newhilb,*new1;
 
   Lm = id_Head(strat->Shdl,currRing);
 
-  newhilb =hHstdSeries(Lm,w,strat->kHomW,Q,currRing); // ,strat->tailRing?
+  newhilb =hFirstSeries(Lm,w,Q,strat->kHomW);
 
   if(newhilb->compare(hilb) == 0)
   {
