@@ -5,11 +5,7 @@
 *  ABSTRACT -  Kernel: factorizing alg. of Buchberger
 */
 
-
-
-
 #include "kernel/mod2.h"
-#include "omalloc/omalloc.h"
 #include "misc/options.h"
 #include "kernel/polys.h"
 #include "kernel/ideals.h"
@@ -22,7 +18,7 @@
 #include "kernel/ideals.h"
 #include "kernel/GBEngine/kstdfac.h"
 
-int strat_nr=0;
+VAR int strat_nr=0;
 /*3
 * copy o->T to n->T, assumes that n->S is already copied
 */
@@ -189,7 +185,6 @@ kStrategy kStratCopy(kStrategy o)
   s->tail = pInit();
   copyL(o,s);//s->L=...
   s->B=initL();
-  s->kHEdge=pCopy(o->kHEdge);
   s->kNoether=pCopy(o->kNoether);
   if (o->NotUsedAxis!=NULL)
   {
@@ -218,22 +213,18 @@ kStrategy kStratCopy(kStrategy o)
   s->syzComp=o->syzComp;
   s->LazyPass=o->LazyPass;
   s->LazyDegree=o->LazyDegree;
-  s->HCord=o->HCord;
   s->lastAxis=o->lastAxis;
   s->interpt=o->interpt;
   s->homog=o->homog;
   s->news=o->news;
   s->newt=o->newt;
-  s->kHEdgeFound=o->kHEdgeFound;
+  s->kAllAxis=o->kAllAxis;
   s->honey=o->honey;
   s->sugarCrit=o->sugarCrit;
   s->Gebauer=o->Gebauer;
   s->noTailReduction=o->noTailReduction;
   s->fromT=o->fromT;
   s->noetherSet=o->noetherSet;
-#ifdef HAVE_SHIFTBBA
-  s->lV=o->lV;
-#endif
 #ifdef HAVE_PLURAL
   s->no_prod_crit=o->no_prod_crit;
 #endif
@@ -574,7 +565,7 @@ ideal bbafac (ideal /*F*/, ideal Q,intvec */*w*/,kStrategy strat, ideal_list FL)
           }
         }
       }
-      if (strat->P.lcm!=NULL) pLmFree(strat->P.lcm);
+      kDeleteLcm(&strat->P);
       int i;
 
       for(i=IDELEMS(fac)-1;i>=0;i--)
