@@ -351,7 +351,7 @@ public:
   int Ll,Lmax;
   int Bl,Bmax;
   int ak,LazyDegree,LazyPass;
-  int syzComp;
+  unsigned syzComp;
   int lastAxis;
   int newIdeal;
   int minim;
@@ -416,7 +416,6 @@ public:
 void deleteHC(poly *p, int *e, int *l, kStrategy strat);
 void deleteHC(LObject* L, kStrategy strat, BOOLEAN fromNext = FALSE);
 void deleteInS (int i,kStrategy strat);
-void deleteInSSba (int i,kStrategy strat);
 void cleanT (kStrategy strat);
 static inline LSet initL (int nr=setmaxL)
 { return (LSet)omAlloc(nr*sizeof(LObject)); }
@@ -461,8 +460,6 @@ int posInLSig (const LSet set, const int length,
                LObject* L,const kStrategy strat);
 int posInLSigRing (const LSet set, const int length,
                LObject* L,const kStrategy strat);
-int posInLRing (const LSet set, const int length,
-               LObject* L,const kStrategy strat);
 int posInSyz (const kStrategy strat, const poly sig);
 int posInL0 (const LSet set, const int length,
              LObject* L,const kStrategy strat);
@@ -493,6 +490,7 @@ KINLINE poly redtailBbaBound (poly p,int end_pos,kStrategy strat,int bound,BOOLE
 #ifdef HAVE_RINGS
 KINLINE poly redtailBba_Ring (poly p,int end_pos,kStrategy strat);
 KINLINE poly redtailBba_Z (poly p,int end_pos,kStrategy strat);
+poly redtailBba_NF (poly p, kStrategy strat );
 poly redtailBba_Ring (LObject* L, int end_pos, kStrategy strat );
 poly redtailBba_Z (LObject* L, int end_pos, kStrategy strat );
 void redtailBbaAlsoLC_Z (LObject* L, int end_pos, kStrategy strat );
@@ -510,6 +508,7 @@ poly redNF (poly h,int & max_ind,int nonorm,kStrategy strat);
 int redNF0 (LObject *P,kStrategy strat);
 poly redNFTail (poly h,const int sl,kStrategy strat);
 int redHoney (LObject* h, kStrategy strat);
+int redHoneyM (LObject* h, kStrategy strat);
 int redLiftstd (LObject* h, kStrategy strat);
 #ifdef HAVE_RINGS
 int redRing (LObject* h,kStrategy strat);
@@ -698,12 +697,14 @@ void f5c (kStrategy strat, int& olddeg, int& minimcnt, int& hilbeledeg,
 //            (strat == NULL)
 //         -1 tailRing change could not be performed due to exceeding exp
 //            bound of currRing
+//  reduce should be set inside "kNF" (only relevant for rings)
 int ksReducePoly(LObject* PR,
                  TObject* PW,
                  poly spNoether = NULL,
                  number *coef = NULL,
                  poly *mon =NULL,
-                 kStrategy strat = NULL);
+                 kStrategy strat = NULL,
+                 BOOLEAN redtail = FALSE);
 
 /* like ksReducePoly, but if the reducer has only 1 term we still
  * compute a possible coefficient multiplier for PR. this comes from
@@ -764,7 +765,7 @@ int ksReducePolySigRing(LObject* PR,
 
 // Reduces PR at Current->next with PW
 // Assumes PR != NULL, Current contained in PR
-//         Current->next != NULL, LM(PW) devides LM(Current->next)
+//         Current->next != NULL, LM(PW) divides LM(Current->next)
 // Changes: PR
 // Const:   PW
 // Return: see ksReducePoly
@@ -856,12 +857,6 @@ static inline int kFindInL1(const poly p, const kStrategy strat)
   return -1;
 }
 
-poly pMove2CurrTail(poly p, kStrategy strat);
-
-poly pMoveCurrTail2poly(poly p, kStrategy strat);
-
-poly pCopyL2p(LObject h, kStrategy strat);
-
 void enterTShift(LObject p, kStrategy strat, int atT = -1);
 
 BOOLEAN enterOnePairShift (poly q, poly p, int ecart, int isFromQ, kStrategy strat, int atR, int ecartq, int qisFromQ, int shiftcount, int ifromS);
