@@ -85,7 +85,7 @@ void addLObject(LObject& h, kStrategy& strat)
     strat->enterS(h, pos, strat, -1);
 //    enterT(h, strat); // ?!
 
-    if (h.lcm!=NULL) pLmFree(h.lcm);
+    kDeleteLcm(&h);
   }
 
 
@@ -457,7 +457,7 @@ ideal k_sca_bba (const ideal F, const ideal Q, const intvec */*w*/, const intvec
     strat->use_buckets = 1;
 #endif
 
-  // redtailBBa against T for inhomogenous input
+  // redtailBBa against T for inhomogeneous input
   if (!TEST_OPT_OLDSTD)
     withT = ! strat->homog;
 
@@ -483,7 +483,7 @@ ideal k_sca_bba (const ideal F, const ideal Q, const intvec */*w*/, const intvec
 
   //  due to std( SB, p).
   // Note that after initBuchMora :: initSSpecial all these additional
-  // elements are in S and T (and some pairs are in L, which also has no initiall
+  // elements are in S and T (and some pairs are in L, which also has no initial
   // elements!!!)
   if(TEST_OPT_SB_1)
   {
@@ -680,7 +680,7 @@ ideal k_sca_bba (const ideal F, const ideal Q, const intvec */*w*/, const intvec
 //       if (hilb!=NULL) khCheck(Q,w,hilb,hilbeledeg,hilbcount,strat);
 
 //      Print("[%d]",hilbeledeg);
-      if (strat->P.lcm!=NULL) pLmFree(strat->P.lcm);
+      kDeleteLcm(&strat->P);
 
       // //////////////////////////////////////////////////////////
       // SCA:
@@ -1108,8 +1108,7 @@ ideal k_sca_mora(const ideal F, const ideal Q, const intvec */*w*/, const intvec
 
 
       // clear strat->P
-      if (strat->P.lcm!=NULL) pLmFree(strat->P.lcm);
-      strat->P.lcm=NULL;
+      kDeleteLcm(&strat->P);
 
       // //////////////////////////////////////////////////////////
       // SCA:
@@ -1180,15 +1179,14 @@ ideal k_sca_mora(const ideal F, const ideal Q, const intvec */*w*/, const intvec
   // - polynomials used for HECKE: HC, noether -
   if (TEST_OPT_FINDET)
   {
-    if (strat->kHEdge!=NULL)
-      Kstd1_mu=currRing->pFDeg(strat->kHEdge,currRing);
+    if (strat->kNoether!=NULL)
+      Kstd1_mu=currRing->pFDeg(strat->kNoether,currRing);
     else
       Kstd1_mu=-1;
   }
-  if(strat->kHEdge!=NULL) pLmFree(&strat->kHEdge);
+  if(strat->kNoether!=NULL) pLmFree(&strat->kNoether);
   strat->update = TRUE; //???
   strat->lastAxis = 0; //???
-  pDelete(&strat->kNoether);
   omFreeSize((ADDRESS)strat->NotUsedAxis,(rVar(currRing)+1)*sizeof(BOOLEAN));
   if (TEST_OPT_PROT) messageStat(hilbcount,strat);
 //  if (TEST_OPT_WEIGHTM)
