@@ -7,7 +7,6 @@
 
 #include "kernel/mod2.h"
 #include "misc/options.h"
-#include "omalloc/omalloc.h"
 #include "kernel/polys.h"
 #include "kernel/GBEngine/kstd1.h"
 #include "kernel/GBEngine/kutil.h"
@@ -1030,6 +1029,7 @@ syStrategy sySchreyer(ideal arg, int maxlength)
   {
     if (fr[i]!=NULL)
     {
+      idSkipZeroes(fr[i]);
       result->fullres[i] = fr[i];
       fr[i] = NULL;
     }
