@@ -385,7 +385,7 @@ long syReorderShiftedComponents(long * sc, int n)
   }
 #endif
 
-  omMemcpyW(sc, tc, n);
+  memcpy(sc, tc, n*sizeof(long));
   omFreeSize(tc, n*sizeof(long));
   return new_space;
 }
@@ -513,7 +513,7 @@ static BOOLEAN syOrder(poly p,syStrategy syzstr,int index,
     if (same_comp == 0)
       same_comp = SYZ_SHIFT_BASE;
 
-    // test wheter we have enough space for new shifted component
+    // test whether we have enough space for new shifted component
     if ((LONG_MAX - same_comp) <= shind[ie-1])
     {
       long new_space = syReorderShiftedComponents(shind, ie);
@@ -1806,7 +1806,7 @@ intvec * syBettiOfComputation(syStrategy syzstr, BOOLEAN minim,int * row_shift,
 
   return result; /// Don't change the syzstr???
 
-  // TODO: cleanup thses!
+  // TODO: cleanup these!
   if( fullres != NULL && syzstr->fullres == NULL )
     syzstr->fullres = fullres;
   if( minres != NULL && syzstr->minres == NULL )
@@ -2170,7 +2170,6 @@ static poly syMinimizeP1(int toMin,syStrategy syzstr,intvec * ordn,int index,
       tempStripped =
         syStripOutCopy(syzstr->res[index+1]->m[sPairs[i].syzind],toStrip);
       tc = pGetComp(sPairs[i].isNotMinimal);
-      //p = pTakeOutComp1(&tempStripped,tc);
       int lu;
       pTakeOutComp(&tempStripped,tc,&p,&lu);
       kBucketTakeOutComp(syzstr->bucket,tc,&mp,&lp);
@@ -2269,7 +2268,7 @@ static intvec * syToStrip(syStrategy syzstr, int index)
 
 /*2
 * re-computes the order of pairs during the algorithm
-* this ensures to procede with a triangular matrix
+* this ensures to proceed with a triangular matrix
 */
 static intvec * syOrdPairs(SSet sPairs, int length)
 {
@@ -2379,7 +2378,7 @@ static resolvente syReadOutMinimalRes(syStrategy syzstr,
   {
     rChangeCurrRing(origR);
     // Thomas: now make sure that all data which you need is pFetchCopied
-    // maybe incoporate it into syReorder ??
+    // maybe incorporate it into syReorder ??
   }
   tres = syReorder(tres,syzstr->length,syzstr,FALSE,syzstr->res);
   syKillEmptyEntres(tres,syzstr->length);
