@@ -24,7 +24,8 @@
 #include "misc/options.h"
 #endif
 
-VAR int  hCo, hMu, hMu2;
+VAR int  hCo, hMu2;
+VAR long hMu;
 VAR omBin indlist_bin = omGetSpecBin(sizeof(indlist));
 
 /*0 implementation*/
@@ -80,7 +81,7 @@ int  scDimInt(ideal S, ideal Q)
   if( Q!=NULL ) id_Test(Q, currRing);
 
   int  mc;
-  hexist = hInit(S, Q, &hNexist, currRing);
+  hexist = hInit(S, Q, &hNexist);
   if (!hNexist)
     return (currRing->N);
   hwork = (scfmon)omAlloc(hNexist * sizeof(scmon));
@@ -289,7 +290,7 @@ intvec * scIndIntvec(ideal S, ideal Q)
 
   intvec *Set=new intvec((currRing->N));
   int  mc,i;
-  hexist = hInit(S, Q, &hNexist, currRing);
+  hexist = hInit(S, Q, &hNexist);
   if (hNexist==0)
   {
     for(i=0; i<(currRing->N); i++)
@@ -374,10 +375,7 @@ static void hIndep(scmon pure)
   Set = ISet->set = new intvec((currRing->N));
   for (iv=(currRing->N); iv!=0 ; iv--)
   {
-    if (pure[iv])
-      (*Set)[iv-1] = 0;
-    else
-      (*Set)[iv-1] = 1;
+    (*Set)[iv-1] = (pure[iv]==0);
   }
   ISet = ISet->nx = (indset)omAlloc0Bin(indlist_bin);
   hMu++;
@@ -557,10 +555,7 @@ static void hCheckIndep(scmon pure)
       Set = res->set;
       for (iv=(currRing->N); iv; iv--)
       {
-        if (pure[iv])
-          (*Set)[iv-1] = 0;
-        else
-          (*Set)[iv-1] = 1;
+        (*Set)[iv-1] = (pure[iv]==0);
       }
     }
   }
@@ -623,9 +618,10 @@ void hIndAllMult(scmon pure, int Npure, scfmon rad, int Nrad,
 
 // multiplicity
 
-static int hZeroMult(scmon pure, scfmon stc, int Nstc, varset var, int Nvar)
+static long hZeroMult(scmon pure, scfmon stc, int Nstc, varset var, int Nvar)
 {
-  int  iv = Nvar -1, sum, a, a0, a1, b, i;
+  int  iv = Nvar -1, a, a0, a1, b, i;
+  long sum;
   int  x, x0;
   scmon pn;
   scfmon sn;
@@ -643,9 +639,28 @@ static int hZeroMult(scmon pure, scfmon stc, int Nstc, varset var, int Nvar)
   sn = hGetmem(Nstc, stc, stcmem[iv]);
   hStepS(sn, Nstc, var, Nvar, &a, &x);
   if (a == Nstc)
-    return pure[var[Nvar]] * hZeroMult(pn, sn, a, var, iv);
+  {
+    #if  SIZEOF_LONG==8
+    return (long)pure[var[Nvar]] * hZeroMult(pn, sn, a, var, iv);
+    #else
+    int64 t=hZeroMult(pn, sn, a, var, iv);
+    t *= pure[var[Nvar]];
+    if ((t>=INT_MIN)&&(t<=INT_MAX)) sum=t;
+    else if (!errorreported) WerrorS("int overflow in vdim 3");
+    return sum;
+    #endif
+  }
   else
+  {
+    #if  SIZEOF_LONG==8
     sum = x * hZeroMult(pn, sn, a, var, iv);
+    #else
+    int64 t=hZeroMult(pn, sn, a, var, iv);
+    t *= x;
+    if ((t>=INT_MIN)&&(t<=INT_MAX)) sum=t;
+    else if (!errorreported) WerrorS("int overflow in vdim 4");
+    #endif
+  }
   b = a;
   loop
   {
@@ -659,11 +674,27 @@ static int hZeroMult(scmon pure, scfmon stc, int Nstc, varset var, int Nvar)
     b += (a1 - a0);
     if (a < Nstc)
     {
-      sum += (x - x0) * hZeroMult(pn, sn, b, var, iv);
+      #if  SIZEOF_LONG==8
+      sum += (long)(x - x0) * hZeroMult(pn, sn, b, var, iv);
+      #else
+      int64 t=hZeroMult(pn, sn, b, var, iv);
+      t *= (x-x0);
+      t += sum;
+      if ((t>=INT_MIN)&&(t<=INT_MAX)) sum=t;
+      else if (!errorreported) WerrorS("int overflow in vdim 1");
+      #endif
     }
     else
     {
-      sum += (pure[var[Nvar]] - x0) * hZeroMult(pn, sn, b, var, iv);
+      #if  SIZEOF_LONG==8
+      sum += (long)(pure[var[Nvar]] - x0) * hZeroMult(pn, sn, b, var, iv);
+      #else
+      int64 t=hZeroMult(pn, sn, b, var, iv);
+      t *= (pure[var[Nvar]]-x0);
+      t += sum;
+      if ((t>=INT_MIN)&&(t<=INT_MAX)) sum=t;
+      else if (!errorreported) WerrorS("int overflow in vdim 2");
+      #endif
       return sum;
     }
   }
@@ -775,7 +806,7 @@ static void hDegree(ideal S, ideal Q)
 
   int  di;
   int  mc;
-  hexist = hInit(S, Q, &hNexist, currRing);
+  hexist = hInit(S, Q, &hNexist);
   if (!hNexist)
   {
     hCo = 0;
@@ -916,17 +947,17 @@ void scDegree(ideal S, intvec *modulweight, ideal Q)
   delete hseries2;
 }
 
-static void hDegree0(ideal S, ideal Q, const ring tailRing)
+long  scMult0Int(ideal S, ideal Q)
 {
-  id_TestTail(S, currRing, tailRing);
-  if (Q!=NULL) id_TestTail(Q, currRing, tailRing);
+  id_LmTest(S, currRing);
+  if (Q!=NULL) id_LmTest(Q, currRing);
 
   int  mc;
-  hexist = hInit(S, Q, &hNexist, tailRing);
+  hexist = hInit(S, Q, &hNexist);
   if (!hNexist)
   {
     hMu = -1;
-    return;
+    return -1;
   }
   else
     hMu = 0;
@@ -988,18 +1019,9 @@ static void hDegree0(ideal S, ideal Q, const ring tailRing)
   hDelete(hexist, hNexist);
   if (hisModule)
     omFreeSize((ADDRESS)hstc, hNexist * sizeof(scmon));
-}
-
-int  scMult0Int(ideal S, ideal Q, const ring tailRing)
-{
-  id_TestTail(S, currRing, tailRing);
-  if (Q!=NULL) id_TestTail(Q, currRing, tailRing);
-
-  hDegree0(S, Q, tailRing);
   return hMu;
 }
 
-
 // HC
 
 STATIC_VAR poly pWork;
@@ -1076,10 +1098,10 @@ static void hHedgeStep(scmon pure, scfmon stc,
   }
 }
 
-void scComputeHC(ideal S, ideal Q, int ak, poly &hEdge, ring tailRing)
+void scComputeHC(ideal S, ideal Q, int ak, poly &hEdge)
 {
-  id_TestTail(S, currRing, tailRing);
-  if (Q!=NULL) id_TestTail(Q, currRing, tailRing);
+  id_LmTest(S, currRing);
+  if (Q!=NULL) id_LmTest(Q, currRing);
 
   int  i;
   int  k = ak;
@@ -1087,17 +1109,17 @@ void scComputeHC(ideal S, ideal Q, int ak, poly &hEdge, ring tailRing)
   if (rField_is_Ring(currRing) && (currRing->OrdSgn == -1))
   {
     //consider just monic generators (over rings with zero-divisors)
-    ideal SS=id_Copy(S,tailRing);
+    ideal SS=id_Head(S,currRing);
     for(i=0;i<=idElem(S);i++)
     {
       if((SS->m[i]!=NULL)
-      && ((p_IsPurePower(SS->m[i],tailRing)==0)
-        ||(!n_IsUnit(pGetCoeff(SS->m[i]), tailRing->cf))))
+      && ((p_IsPurePower(SS->m[i],currRing)==0)
+        ||(!n_IsUnit(pGetCoeff(SS->m[i]), currRing->cf))))
       {
-        p_Delete(&SS->m[i],tailRing);
+        p_Delete(&SS->m[i],currRing);
       }
     }
-    S=id_Copy(SS,tailRing);
+    S=id_Copy(SS,currRing);
     idSkipZeroes(S);
   }
   #if 0
@@ -1112,7 +1134,7 @@ void scComputeHC(ideal S, ideal Q, int ak, poly &hEdge, ring tailRing)
   if(idElem(S) == 0)
     return;
   hNvar = (currRing->N);
-  hexist = hInit(S, Q, &hNexist, tailRing); // tailRing?
+  hexist = hInit(S, Q, &hNexist);
   if (k!=0)
     hComp(hexist, hNexist, k, hexist, &hNstc);
   else
@@ -1441,7 +1463,7 @@ ideal scKBase(int deg, ideal s, ideal Q, intvec * mv)
     }
   }
   stcmem = hCreate((currRing->N) - 1);
-  hexist = hInit(s, Q, &hNexist, currRing);
+  hexist = hInit(s, Q, &hNexist);
   p = last = pInit();
   /*pNext(p) = NULL;*/
   act = (scmon)omAlloc(((currRing->N) + 1) * sizeof(int));
@@ -1492,10 +1514,10 @@ ende:
 
 #if 0 //-- alternative implementation of scComputeHC
 /*
-void scComputeHCw(ideal ss, ideal Q, int ak, poly &hEdge, ring tailRing)
+void scComputeHCw(ideal ss, ideal Q, int ak, poly &hEdge)
 {
-  id_TestTail(ss, currRing, tailRing);
-  if (Q!=NULL) id_TestTail(Q, currRing, tailRing);
+  id_LmTest(ss, currRing);
+  if (Q!=NULL) id_LmTest(Q, currRing);
 
   int  i, di;
   poly p;
@@ -1510,7 +1532,7 @@ void scComputeHCw(ideal ss, ideal Q, int ak, poly &hEdge, ring tailRing)
   }
   di = scDimInt(s, Q);
   stcmem = hCreate((currRing->N) - 1);
-  hexist = hInit(s, Q, &hNexist, currRing);
+  hexist = hInit(s, Q, &hNexist);
   p = last = pInit();
   // pNext(p) = NULL;
   act = (scmon)omAlloc(((currRing->N) + 1) * sizeof(int));
