@@ -11,9 +11,9 @@
 #include "kernel/polys.h"
 #include "misc/intvec.h"
 
-void scComputeHC(ideal s,ideal Q, int  k,poly &hEdge, ring tailRing = currRing);
+void scComputeHC(ideal s,ideal Q, int  k,poly &hEdge);
 #if 0 // - alternative implementation for tests
-void scComputeHCw(ideal s,ideal Q, int  k,poly &hEdge, ring tailRing = currRing);
+void scComputeHCw(ideal s,ideal Q, int  k,poly &hEdge);
 #endif
 
 intvec * scIndIntvec(ideal S, ideal Q=NULL);
@@ -21,11 +21,11 @@ intvec * scIndIntvec(ideal S, ideal Q=NULL);
 // lists scIndIndset(ideal S, BOOLEAN all, ideal Q=NULL); // TODO: move to Singular/
 
 /// ideal dimension
-int scDimInt(ideal  s,ideal Q=NULL);
+int scDimInt(ideal  s,ideal Q=NULL); // dim (cf: field)
 /// scDimInt for ring-coefficients
-int scDimIntRing(ideal  s,ideal Q=NULL);
-int scMultInt(ideal  s,ideal Q=NULL);
-int scMult0Int(ideal  s,ideal Q=NULL, const ring tailRing = currRing);
+int scDimIntRing(ideal  s,ideal Q=NULL); // dim (general)
+int scMultInt(ideal  s,ideal Q=NULL); // mult
+long scMult0Int(ideal  s,ideal Q=NULL); // vdim
 void scPrintDegree(int co, int mu);
 void scDegree(ideal  s,intvec *modulweight,ideal Q=NULL);
 
