@@ -558,7 +558,7 @@ ideal idMultSect(resolvente arg, int length, GbVariant alg)
   if(syz_ring!=orig_ring)
     rChangeCurrRing(orig_ring);
 
-  /* interprete result ----------------------------------------*/
+  /* interpret result ----------------------------------------*/
   result = idInit(IDELEMS(tempstd),maxrk);
   k = 0;
   for (j=0;j<IDELEMS(tempstd);j++)
@@ -1630,11 +1630,7 @@ ideal idElimination (ideal h1,poly delVar,intvec *hilb, GbVariant alg)
 #endif
   hom=(tHomog)idHomModule(h1,NULL,&w); //sets w to weight vector or NULL
   h3=idInit(16,h1->rank);
-  for (k=0;; k++)
-  {
-    if (origR->order[k]!=0) ordersize++;
-    else break;
-  }
+  ordersize=rBlocks(origR)+1;
 #if 0
   if (rIsPluralRing(origR)) // we have too keep the odering: it may be needed
                             // for G-algebra
@@ -1733,7 +1729,17 @@ ideal idElimination (ideal h1,poly delVar,intvec *hilb, GbVariant alg)
       block0[k+1] = origR->block0[k];
       block1[k+1] = origR->block1[k];
       ord[k+1] = origR->order[k];
-      if (origR->wvhdl[k]!=NULL) wv[k+1] = (int*) omMemDup(origR->wvhdl[k]);
+      if (origR->wvhdl[k]!=NULL)
+      #ifdef HAVE_OMALLOC
+        wv[k+1] = (int*) omMemDup(origR->wvhdl[k]);
+      #else
+      {
+        int l=(origR->block1[k]-origR->block0[k]+1)*sizeof(int);
+        if (origR->order[k]==ringorder_a64) l*=2;
+        wv[k+1]=(int*)omalloc(l);
+        memcpy(wv[k+1],origR->wvhdl[k],l);
+      }
+      #endif
     }
     block0[0] = 1;
     block1[0] = rVar(origR);
@@ -2157,7 +2163,7 @@ matrix idDiffOp(ideal I, ideal J,BOOLEAN multiply)
 }
 
 /*3
-*handles for some ideal operations the ring/syzcomp managment
+*handles for some ideal operations the ring/syzcomp management
 *returns all syzygies (componentwise-)shifted by -syzcomp
 *or -syzcomp-1 (in case of ideals as input)
 static ideal idHandleIdealOp(ideal arg,int syzcomp,int isIdeal=FALSE)
@@ -2576,7 +2582,7 @@ int idIndexOfKBase(poly monom, ideal kbase)
 
 /*2
 *decomposes the monom in a part of coefficients described by the
-*complement of how and a monom in variables occuring in how, the
+*complement of how and a monom in variables occurring in how, the
 *index of which in kbase is returned as integer pos (-1 if it don't
 *exists)
 */
