@@ -12,12 +12,12 @@
 # include "singularconfig.h"
 # include "misc/auxiliary.h"
 
+#if defined(HAVE_MPFR_H) && defined(HAVE_LIBMPFR)
 #define HAVE_SVD
+#endif
 
-#undef VERSION
-#define VERSION "4.1.1"
 #define SINGULAR_MAJOR_VERSION 4
-#define SINGULAR_MINOR_VERSION 1
+#define SINGULAR_MINOR_VERSION 3
 #define SINGULAR_SUB_VERSION 1
 #define S_ROOT_DIR ""
 
@@ -52,20 +52,19 @@
 /* linear algebra extensions from pcv.h/pcv.cc */
 #define HAVE_PCV 1
 
-/* procedures to compute groebner bases with the f5 implementation */
-/* still testing */
-#undef HAVE_F5
-
-/* procedures to compute groebner bases with the f5c implementation */
-/* still testing */
-#undef HAVE_F5C
-
 /* procedures to compute with units */
 #define HAVE_UNITS
 
 /* Define to use scanner when loading libraries */
 #define HAVE_LIBPARSER
 
+/* define for parallel processes with shared memory */
+#ifndef __CCYGWIN__
+#if !defined(__MACH__) || !defined(__APPLE__)
+#define HAVE_VSPACE 1
+#endif
+#endif
+
 /*#define PROFILING*/
 #ifdef PROFILING
 #define PROFILER ,0,0
