@@ -196,10 +196,10 @@ bigintmat * bimAdd(bigintmat * a, bigintmat * b)
 
   return bim;
 }
-bigintmat * bimAdd(bigintmat * a, int b)
+bigintmat * bimAdd(bigintmat * a, long b)
 {
 
-  const int mn = a->rows()*a->cols();
+  const int mn = si_min(a->rows(),a->cols());
 
   const coeffs basecoeffs = a->basecoeffs();
   number bb=n_Init(b,basecoeffs);
@@ -208,8 +208,8 @@ bigintmat * bimAdd(bigintmat * a, int b)
 
   bigintmat * bim = new bigintmat(a->rows(),a->cols() , basecoeffs);
 
-  for (i=0; i<mn; i++)
-    bim->rawset(i, n_Add((*a)[i], bb, basecoeffs), basecoeffs);
+  for (i=1; i<=mn; i++)
+    BIMATELEM(*bim,i,i)=n_Add(BIMATELEM(*a,i,i), bb, basecoeffs);
 
   n_Delete(&bb,basecoeffs);
   return bim;
@@ -233,9 +233,9 @@ bigintmat * bimSub(bigintmat * a, bigintmat * b)
   return bim;
 }
 
-bigintmat * bimSub(bigintmat * a, int b)
+bigintmat * bimSub(bigintmat * a, long b)
 {
-  const int mn = a->rows()*a->cols();
+  const int mn = si_min(a->rows(),a->cols());
 
   const coeffs basecoeffs = a->basecoeffs();
   number bb=n_Init(b,basecoeffs);
@@ -244,8 +244,8 @@ bigintmat * bimSub(bigintmat * a, int b)
 
   bigintmat * bim = new bigintmat(a->rows(),a->cols() , basecoeffs);
 
-  for (i=0; i<mn; i++)
-    bim->rawset(i, n_Sub((*a)[i], bb, basecoeffs), basecoeffs);
+  for (i=1; i<=mn; i++)
+    BIMATELEM(*bim,i,i)=n_Sub(BIMATELEM(*a,i,i), bb, basecoeffs);
 
   n_Delete(&bb,basecoeffs);
   return bim;
@@ -298,7 +298,7 @@ bigintmat * bimMult(bigintmat * a, bigintmat * b)
   return bim;
 }
 
-bigintmat * bimMult(bigintmat * a, int b)
+bigintmat * bimMult(bigintmat * a, long b)
 {
 
   const int mn = a->rows()*a->cols();
@@ -592,9 +592,8 @@ int * bigintmat::getwid(int maxwid)
       char * tmp = StringEndS();
       const int _nl = strlen(tmp);
       wv[col*i+j] = _nl;
-      if (_nl > cwv[j])
-        cwv[j]=_nl;
-        omFree(tmp);
+      if (_nl > cwv[j]) cwv[j]=_nl;
+      omFree(tmp);
     }
   }
 
