@@ -22,14 +22,9 @@
 #include "coeffs/modulop.h"
 #include "coeffs/shortfl.h"
 
-#ifdef LDEBUG
-BOOLEAN  ngcDBTest(number a, const char *f, const int l, const coeffs r);
-#endif
-
-
 #ifdef LDEBUG
 // not yet implemented
-BOOLEAN ngcDBTest(number, const char *, const int, const coeffs r)
+static BOOLEAN ngcDBTest(number, const char *, const int, const coeffs r)
 {
   assume( getCoeffType(r) == n_long_C );
 
@@ -161,6 +156,13 @@ static number ngcAdd (number a, number b, const coeffs R)
   return (number)r;
 }
 
+static void ngcInpAdd (number &a, number b, const coeffs R)
+{
+  assume( getCoeffType(R) == n_long_C );
+
+  (*(gmp_complex*)a) += (*(gmp_complex*)b);
+}
+
 /*2
 * u:= a - b
 */
@@ -183,6 +185,13 @@ static number ngcMult (number a, number b, const coeffs R)
   return (number)r;
 }
 
+static void ngcInpMult (number &a, number b, const coeffs R)
+{
+  assume( getCoeffType(R) == n_long_C );
+
+  (*(gmp_complex*)a) *= (*(gmp_complex*)b);
+}
+
 /*2
 * u := a / b
 */
@@ -215,7 +224,6 @@ static void ngcPower ( number x, int exp, number * u, const coeffs r)
   }
   else if ( exp == 1 )
   {
-    n_New(u, r);
     gmp_complex* n = new gmp_complex();
     *n= *(gmp_complex*)x;
     *u=(number)n;
@@ -223,7 +231,6 @@ static void ngcPower ( number x, int exp, number * u, const coeffs r)
   }
   else if (exp == 2)
   {
-    n_New(u, r);
     gmp_complex* n = new gmp_complex();
     *n= *(gmp_complex*)x;
     *u=(number)n;
@@ -240,8 +247,7 @@ static void ngcPower ( number x, int exp, number * u, const coeffs r)
   }
   else
   {
-    number w;
-    n_New(&w, r);
+    number w=NULL;
     ngcPower(x,exp/2,&w, r);
     ngcPower(w,2,u, r);
     n_Delete(&w, r);
@@ -569,8 +575,10 @@ BOOLEAN ngcInitChar(coeffs n, void* parameter)
   n->cfInitMPZ = ngcInitMPZ;
   n->cfInt     = ngcInt;
   n->cfAdd     = ngcAdd;
+  n->cfInpAdd  = ngcInpAdd;
   n->cfSub     = ngcSub;
   n->cfMult    = ngcMult;
+  n->cfInpMult = ngcInpMult;
   n->cfDiv     = ngcDiv;
   n->cfExactDiv= ngcDiv;
   n->cfInpNeg     = ngcNeg;
@@ -582,6 +590,7 @@ BOOLEAN ngcInitChar(coeffs n, void* parameter)
   n->cfIsOne   = ngcIsOne;
   n->cfIsMOne  = ngcIsMOne;
   n->cfGreaterZero = ngcGreaterZero;
+  n->cfSize    = ngcSize;
 
   n->cfWriteLong  = ngcWrite;
   n->cfWriteShort = ngcWrite;
@@ -645,7 +654,6 @@ BOOLEAN ngcInitChar(coeffs n, void* parameter)
   r->cfSetMap = nlSetMap;
   r->cfName = ndName;
   r->cfInpMult=nlInpMult;
-  r->cfInit_bigint=nlCopyMap;
 #ifdef LDEBUG
   // debug stuff
   r->cfDBTest=nlDBTest;
@@ -694,6 +702,3 @@ void ngcSetChar(const coeffs r)
 {
   setGMPFloatDigits(r->float_len, r->float_len2);
 }
-
-
-
