@@ -21,8 +21,6 @@
 #include "coeffs/gnumpc.h"
 #include "coeffs/modulop.h"
 
-const char *   ngfRead (const char *s, number *a, const coeffs r);
-
 union nf
 {
   SI_FLOAT _f;
@@ -158,6 +156,13 @@ static number ngfAdd (number a, number b, const coeffs R)
   return (number)r;
 }
 
+static void ngfInpAdd (number &a, number b, const coeffs R)
+{
+  assume( getCoeffType(R) == n_long_R );
+
+  (*(gmp_float*)a) += (*(gmp_float*)b);
+}
+
 /*2
 * u:= a - b
 */
@@ -180,6 +185,13 @@ static number ngfMult (number a, number b, const coeffs R)
   return (number)r;
 }
 
+static void ngfInpMult (number &a, number b, const coeffs R)
+{
+  assume( getCoeffType(R) == n_long_R );
+
+  (*(gmp_float*)a) *= (*(gmp_float*)b);
+}
+
 /*2
 * u := a / b
 */
@@ -516,13 +528,15 @@ BOOLEAN ngfInitChar(coeffs n, void *parameter)
   n->cfInitMPZ = ngfInitMPZ;
   n->cfInt     = ngfInt;
   n->cfAdd     = ngfAdd;
+  n->cfInpAdd  = ngfInpAdd;
   n->cfSub     = ngfSub;
   n->cfMult    = ngfMult;
+  n->cfInpMult = ngfInpMult;
   n->cfDiv     = ngfDiv;
   n->cfExactDiv= ngfDiv;
-  n->cfInpNeg     = ngfNeg;
+  n->cfInpNeg  = ngfNeg;
   n->cfInvers  = ngfInvers;
-  n->cfCopy   = ngfCopy;
+  n->cfCopy    = ngfCopy;
   n->cfGreater = ngfGreater;
   n->cfEqual   = ngfEqual;
   n->cfIsZero  = ngfIsZero;
@@ -533,6 +547,7 @@ BOOLEAN ngfInitChar(coeffs n, void *parameter)
   n->cfRead    = ngfRead;
   n->cfPower   = ngfPower;
   n->cfSetMap = ngfSetMap;
+  n->cfSize   = ngfSize;
 #ifdef LDEBUG
   //n->cfDBTest  = ndDBTest; // not yet implemented: ngfDBTest
 #endif
