@@ -72,6 +72,20 @@ static number nnMult (number a,number b, const coeffs r)
   }
 }
 
+static void nnInpMult (number &a,number b, const coeffs r)
+{
+  number_array A=(number_array)a;
+  number_array B=(number_array)b;
+  coeffs_array d=(coeffs_array)r->data;
+  int i=0;
+  loop
+  {
+    d[i]->cfInpMult(A[i],B[i],d[i]);
+    i++;
+    if (d[i]==NULL) return;
+  }
+}
+
 /*2
 * create a number from int
 */
@@ -169,6 +183,20 @@ static number nnAdd (number a, number b, const coeffs r)
   }
 }
 
+static void nnInpAdd (number &a, number b, const coeffs r)
+{
+  number_array A=(number_array)a;
+  number_array B=(number_array)b;
+  coeffs_array d=(coeffs_array)r->data;
+  int i=0;
+  loop
+  {
+    d[i]->cfInpAdd(A[i],B[i],d[i]);
+    i++;
+    if (d[i]==NULL) return;
+  }
+}
+
 static number nnSub (number a, number b, const coeffs r)
 {
   number_array A=(number_array)a;
@@ -361,7 +389,7 @@ static void nnWriteLong (number a, const coeffs r)
 }
 
 #if 0
-void nrPower (number a, int i, number * result, const coeffs r)
+static void nrPower (number a, int i, number * result, const coeffs r)
 {
   assume( getCoeffType(r) == n_R );
 
@@ -481,7 +509,7 @@ static const char * nnRead (const char *s, number *a, const coeffs r)
 /*2
 * test valid numbers: not implemented yet
 */
-BOOLEAN  nnDBTest(number a, const char *f, const int l, const coeffs r)
+static BOOLEAN  nnDBTest(number a, const char *f, const int l, const coeffs r)
 {
   BOOLEAN bo=FALSE;
   number_array A=(number_array)a;
@@ -518,7 +546,7 @@ static number nnMap(number from, const coeffs aRing, const coeffs r)
   }
 }
 
-nMapFunc nnSetMap(const coeffs, const coeffs)
+static nMapFunc nnSetMap(const coeffs, const coeffs)
 {
   return nnMap;
 }
@@ -600,8 +628,10 @@ BOOLEAN nnInitChar(coeffs n, void* p)
   n->cfDelete = nnDelete;
   n->cfInt  = nnInt;
   n->cfAdd  = nnAdd;
+  n->cfInpAdd  = nnInpAdd;
   n->cfSub  = nnSub;
   n->cfMult = nnMult;
+  n->cfInpMult = nnInpMult;
   n->cfDiv  = nnDiv;
   n->cfExactDiv= nnDiv;
   n->cfInpNeg   = nnInpNeg;
