@@ -44,11 +44,8 @@
 
 VAR n_Procs_s *cf_root=NULL;
 
-void   nNew(number* d) { *d=NULL; }
-
-
 static void   ndDelete(number* d, const coeffs) { *d=NULL; }
-static number ndAnn(number, const coeffs) { return NULL;}
+static number ndAnn(number, const coeffs cf) { WarnS("cfAnn undefined"); return n_Init(0,cf); }
 static char* ndCoeffString(const coeffs r)
 {
   return omStrDup(r->cfCoeffName(r));
@@ -113,6 +110,14 @@ static number ndInvers(number a, const coeffs r)
   r->cfDelete(&one,r);
   return res;
 }
+static number ndInvers_Ring(number a, const coeffs r)
+{
+  if (!r->cfIsUnit(a,r)) Print("ndInvers_Ring used with non-unit\n");
+  number one=r->cfInit(1,r);
+  number res=r->cfDiv(one,a,r);
+  r->cfDelete(&one,r);
+  return res;
+}
 
 static BOOLEAN ndIsUnit_Ring(number a, const coeffs r)
 { return r->cfIsOne(a,r)|| r->cfIsMOne(a,r); }
@@ -120,6 +125,10 @@ static BOOLEAN ndIsUnit_Field(number a, const coeffs r)
 { return !r->cfIsZero(a,r); }
 static number ndGetUnit_Ring(number, const coeffs r)
 { return r->cfInit(1,r); }
+static number ndRandom(siRandProc p, number, number, const coeffs cf)
+{ return cf->cfInit(p(),cf); }
+static number ndEucNorm(number a, const coeffs cf)
+{ return cf->cfInit(cf->cfSize(a,cf),cf); }
 #ifdef LDEBUG
 // static void   nDBDummy1(number* d,char *, int) { *d=NULL; }
 static BOOLEAN ndDBTest(number, const char *, const int, const coeffs){ return TRUE; }
@@ -130,11 +139,26 @@ static number ndFarey(number,number,const coeffs r)
   Werror("farey not implemented for %s (c=%d)",r->cfCoeffName(r),getCoeffType(r));
   return NULL;
 }
+static number ndXExtGcd(number, number, number *, number *, number *, number *, const coeffs r)
+{
+  Werror("XExtGcd not implemented for %s (c=%d)",r->cfCoeffName(r),getCoeffType(r));
+  return NULL;
+}
 static number ndChineseRemainder(number *,number *,int,BOOLEAN,CFArray&,const coeffs r)
 {
   Werror("ChineseRemainder not implemented for %s (c=%d)",r->cfCoeffName(r),getCoeffType(r));
   return r->cfInit(0,r);
 }
+number ndReadFd( const ssiInfo *, const coeffs r)
+{
+  Warn("ReadFd not implemented for %s (c=%d)",r->cfCoeffName(r),getCoeffType(r));
+  return NULL;
+}
+
+static void ndWriteFd(number, const ssiInfo *, const coeffs r)
+{
+  Warn("WriteFd not implemented for %s (c=%d)",r->cfCoeffName(r),getCoeffType(r));
+}
 
 static int ndParDeg(number n, const coeffs r)
 {
@@ -163,7 +187,20 @@ BOOLEAN n_IsZeroDivisor( number a, const coeffs r)
 void   ndNormalize(number&, const coeffs) { }
 static number ndReturn0(number, const coeffs r)        { return r->cfInit(0,r); }
 number ndGcd(number, number, const coeffs r)    { return r->cfInit(1,r); }
-static number ndIntMod(number, number, const coeffs r) { return r->cfInit(0,r); }
+static number ndIntMod(number a, number b, const coeffs R)
+{
+  if (R->is_field)
+    return R->cfInit(0,R);
+  else // implementation for a non-field:
+  {
+    number d=n_Div(a,b,R);
+    number p=n_Mult(b,d,R);
+    number r=n_Sub(a,p,R);
+    n_Delete(&p,R);
+    n_Delete(&d,R);
+    return r;
+  }
+}
 static number ndGetDenom(number &, const coeffs r)     { return r->cfInit(1,r); }
 static number ndGetNumerator(number &a,const coeffs r) { return r->cfCopy(a,r); }
 static int    ndSize(number a, const coeffs r)         { return (int)r->cfIsZero(a,r)==FALSE; }
@@ -251,7 +288,6 @@ static void ndClearDenominators(ICoeffsEnumerator& /*numberCollectionEnumerator*
   d = n_Init(1, r);
 }
 
-static number ndCopy(number a, const coeffs) { return a; }
 number ndCopyMap(number a, const coeffs aRing, const coeffs r)
 {
   // aRing and r need not be the same, but must be the same representation
@@ -265,6 +301,7 @@ number ndCopyMap(number a, const coeffs aRing, const coeffs r)
 static void ndKillChar(coeffs) {}
 static void ndSetChar(const coeffs) {}
 
+static number ndCopy(number a, const coeffs) { return a; }
 number nd_Copy(number a, const coeffs r) { return r->cfCopy(a, r); }
 
 #ifdef HAVE_RINGS
@@ -299,6 +336,17 @@ static number ndInitMPZ(mpz_t m, const coeffs r)
   return r->cfInit( mpz_get_si(m), r);
 }
 
+static const char *ndRead(const char * s, number *, const coeffs r)
+{
+  Werror("cfRead is undefined for %s",nCoeffString(r));
+  return s;
+}
+static nMapFunc ndSetMap(const coeffs src, const coeffs dst)
+{
+  if (src==dst) return ndCopyMap;
+  Werror("cfSetMap is undefined for %s",nCoeffString(dst));
+  return NULL;
+}
 
 static BOOLEAN ndCoeffIsEqual(const coeffs r, n_coeffType n, void *)
 {
@@ -307,12 +355,23 @@ static BOOLEAN ndCoeffIsEqual(const coeffs r, n_coeffType n, void *)
   return (n==r->type);
 }
 
-static number ndQuotRem (number a, number b, number * r, const coeffs R)
+number ndQuotRem (number a, number b, number * r, const coeffs R)
 {
   // implementation for a field: r: 0, result: n_Div
-  assume(R->is_field);
-  *r=n_Init(0,R);
-  return n_Div(a,b,R);
+  if(R->is_field)
+  {
+    *r=n_Init(0,R);
+    return n_Div(a,b,R);
+  }
+  else
+  // implementation for a non-field:
+  {
+    number d=n_Div(a,b,R);
+    number p=n_Mult(b,d,R);
+    *r=n_Sub(a,p,R);
+    n_Delete(&p,R);
+    return d;
+  }
 }
 STATIC_VAR n_coeffType nLastCoeffs=n_CF;
 VAR cfInitCharProc nInitCharTableDefault[]=
@@ -377,8 +436,8 @@ coeffs nInitChar(n_coeffType t, void * parameter)
     n->cfCoeffString = ndCoeffString;
     n->cfCoeffWrite = ndCoeffWrite;
     n->cfCoeffName = ndCoeffName; // should alway be changed!
-    n->cfInpMult=ndInpMult;
     n->cfInpAdd=ndInpAdd;
+    n->cfInpMult=ndInpMult;
     n->cfCopy = ndCopy;
     n->cfIntMod=ndIntMod; /* dummy !! */
     n->cfNormalize=ndNormalize;
@@ -390,28 +449,36 @@ coeffs nInitChar(n_coeffType t, void * parameter)
     n->cfPower = ndPower;
     n->cfQuotRem = ndQuotRem;
     n->cfInvers = ndInvers;
+    n->cfRandom = ndRandom;
 
     n->cfKillChar = ndKillChar; /* dummy */
     n->cfSetChar = ndSetChar; /* dummy */
     // temp. removed to catch all the coeffs which miss to implement this!
 
-    n->cfChineseRemainder = ndChineseRemainder;
-    n->cfFarey = ndFarey;
-    n->cfParDeg = ndParDeg;
+    n->cfChineseRemainder = ndChineseRemainder; /* not implemented */
+    n->cfFarey = ndFarey; /* not implemented */
+    n->cfParDeg = ndParDeg; /* not implemented */
+    n->cfReadFd = ndReadFd; /* not implemented */
+    n->cfWriteFd = ndWriteFd; /* not implemented */
 
     n->cfParameter = ndParameter;
 
     n->cfClearContent = ndClearContent;
     n->cfClearDenominators = ndClearDenominators;
 
-    //n->cfIsUnit = ndIsUnit;
+    n->cfEucNorm = ndEucNorm;
 #ifdef HAVE_RINGS
     n->cfDivComp = ndDivComp;
     n->cfDivBy = ndDivBy;
     n->cfExtGcd = ndExtGcd;
+    n->cfXExtGcd = ndXExtGcd;
     //n->cfGetUnit = ndGetUnit_Ring;// set afterwards
 #endif
 
+    // report error, if not redefined
+    n->cfRead=ndRead;
+    n->cfSetMap=ndSetMap;
+
 #ifdef LDEBUG
     n->cfDBTest=ndDBTest;
 #endif
@@ -435,6 +502,7 @@ coeffs nInitChar(n_coeffType t, void * parameter)
     if (n->cfRePart==NULL) n->cfRePart=n->cfCopy;
     if (n->cfExactDiv==NULL) n->cfExactDiv=n->cfDiv;
     if (n->cfSubringGcd==NULL) n->cfSubringGcd=n->cfGcd;
+    if (n->cfWriteShort==NULL) n->cfWriteShort = n->cfWriteLong;
     if (n->cfIsUnit==NULL)
     {
       if (n->is_field) n->cfIsUnit=ndIsUnit_Field;
@@ -446,69 +514,47 @@ coeffs nInitChar(n_coeffType t, void * parameter)
       if (n->is_field) n->cfGetUnit=n->cfCopy;
       else             n->cfGetUnit=ndGetUnit_Ring;
     }
+    if ((n->cfInvers==ndInvers)&&(n->is_field))
+    {
+      n->cfInvers=ndInvers_Ring;
+    }
     #endif
 
-    if(n->cfWriteShort==NULL)
-      n->cfWriteShort = n->cfWriteLong;
 
-    assume(n->nCoeffIsEqual!=NULL);
-    assume(n->cfSetChar!=NULL);
-    assume(n->cfCoeffName!=ndCoeffName);
-    assume(n->cfMult!=NULL);
-    assume(n->cfSub!=NULL);
-    assume(n->cfAdd!=NULL);
-    assume(n->cfDiv!=NULL);
-    assume(n->cfIntMod!=NULL);
-    assume(n->cfExactDiv!=NULL);
-    assume(n->cfInit!=NULL);
-    assume(n->cfInitMPZ!=NULL);
-    assume(n->cfSize!=NULL);
-    assume(n->cfInt!=NULL);
-    assume(n->cfMPZ!=NULL);
-    //assume(n->n->cfDivComp!=NULL);
-    //assume(n->cfIsUnit!=NULL);
-    //assume(n->cfGetUnit!=NULL);
-    //assume(n->cfExtGcd!=NULL);
-    assume(n->cfInpNeg!=NULL);
-    assume(n->cfCopy!=NULL);
-
-    assume(n->cfWriteLong!=NULL);
-    assume(n->cfWriteShort!=NULL);
+    if(n->cfMult==NULL)  PrintS("cfMult missing\n");
+    if(n->cfSub==NULL)  PrintS("cfSub missing\n");
+    if(n->cfAdd==NULL) PrintS("cfAdd missing\n");
+    if(n->cfDiv==NULL) PrintS("cfDiv missing\n");
+    if(n->cfExactDiv==NULL) PrintS("cfExactDiv missing\n");
+    if(n->cfInit==NULL) PrintS("cfInit missing\n");
+    if(n->cfInt==NULL) PrintS("cfInt missing\n");
+    if(n->cfIsUnit==NULL) PrintS("cfIsUnit missing\n");
+    if(n->cfGetUnit==NULL) PrintS("cfGetUnit missing\n");
+    if(n->cfInpNeg==NULL)  PrintS("cfInpNeg missing\n");
+    if(n->cfXExtGcd==NULL)  PrintS("cfXExtGcd missing\n");
+    if(n->cfAnn==NULL)  PrintS("cfAnn missing\n");
+    if(n->cfWriteLong==NULL) PrintS("cfWriteLong missing\n");
 
     assume(n->iNumberOfParameters>= 0);
 
     assume( (n->iNumberOfParameters == 0 && n->pParameterNames == NULL) ||
             (n->iNumberOfParameters >  0 && n->pParameterNames != NULL) );
 
-    assume(n->cfParameter!=NULL);
-    assume(n->cfParDeg!=NULL);
 
-    assume(n->cfRead!=NULL);
-    assume(n->cfNormalize!=NULL);
-    assume(n->cfGreater!=NULL);
-    //assume(n->cfDivBy!=NULL);
-    assume(n->cfEqual!=NULL);
-    assume(n->cfIsZero!=NULL);
-    assume(n->cfIsOne!=NULL);
-    assume(n->cfIsMOne!=NULL);
-    assume(n->cfGreaterZero!=NULL);
-    assume(n->cfGetDenom!=NULL);
-    assume(n->cfGetNumerator!=NULL);
-    assume(n->cfGcd!=NULL);
-    assume(n->cfNormalizeHelper!=NULL);
-    assume(n->cfDelete!=NULL);
-    assume(n->cfSetMap!=NULL);
-    assume(n->cfInpMult!=NULL);
-//    assume(n->cfInit_bigint!=NULL);
-    assume(n->cfCoeffWrite != NULL);
-
-    assume(n->cfClearContent != NULL);
-    assume(n->cfClearDenominators != NULL);
+    if(n->cfGreater==NULL) PrintS("cfGreater missing\n");
+    if(n->cfEqual==NULL) PrintS("cfEqual missing\n");
+    if(n->cfIsZero==NULL) PrintS("cfIsZero missing\n");
+    if(n->cfIsOne==NULL) PrintS("cfIsOne missing\n");
+    if(n->cfIsMOne==NULL) PrintS("cfIsMOne missing\n");
+    if(n->cfGreaterZero==NULL) PrintS("cfGreaterZero missing\n");
+    /* error reporter:
+    if(n->cfRead==ndRead) PrintS("cfRead missing\n");
+    if(n->cfSetMap==ndSetMap) PrintS("cfSetMap missing\n");
+    */
 
     assume(n->type==t);
 
 #ifndef SING_NDEBUG
-    if(n->cfKillChar==NULL) Warn("cfKillChar is NULL for coeff %d",t);
     if(n->cfWriteLong==NULL) Warn("cfWrite is NULL for coeff %d",t);
     if(n->cfWriteShort==NULL) Warn("cfWriteShort is NULL for coeff %d",t);
 #endif
@@ -535,7 +581,7 @@ void nKillChar(coeffs r)
       {
         n->next=n->next->next;
         if (cf_root==r) cf_root=n->next;
-        assume (r->cfKillChar!=NULL); r->cfKillChar(r); 
+        assume (r->cfKillChar!=NULL); r->cfKillChar(r);
         omFreeSize((void *)r, sizeof(n_Procs_s));
         r=NULL;
       }
