@@ -7,7 +7,6 @@
 * ABSTRACT: compatility interface to coeffs
 */
 #include "coeffs/coeffs.h"
-#include "omalloc/omalloc.h" /* for SIZEOF_DOUBLE, SIZEOF_LONG*/
 
 // the access methods
 //
@@ -63,7 +62,6 @@
 // void ndDelete(number* d, const coeffs r);
 number ndGcd(number a, number b, const coeffs);
 // number ndCopy(number a, const coeffs r);
-number ndCopyMap(number a, const coeffs src, const coeffs dst);
 // int ndSize(number a, const coeffs r);
 // number ndGetDenom(number &n, const coeffs r);
 // number ndGetNumerator(number &a,const coeffs r);
@@ -72,6 +70,7 @@ number ndCopyMap(number a, const coeffs src, const coeffs dst);
 
 // void   ndInpMult(number &a, number b, const coeffs r);
 // void   ndInpAdd(number &a, number b, const coeffs r);
+number ndQuotRem (number a, number b, number * r, const coeffs R);
 
 // void ndKillChar(coeffs);
 
@@ -80,6 +79,7 @@ number ndCopyMap(number a, const coeffs src, const coeffs dst);
 // BOOLEAN ndCoeffIsEqual(const coeffs r, n_coeffType n, void * parameter);
 CanonicalForm ndConvSingNFactoryN( number, BOOLEAN /*setChar*/, const coeffs);
 
+number ndReadFd( const ssiInfo *f, const coeffs r);
 /// Test whether a is a zero divisor in r
 /// i.e. not coprime with char. of r
 /// very inefficient implementation:
@@ -89,14 +89,18 @@ BOOLEAN n_IsZeroDivisor( number a, const coeffs r);
 const char* const nDivBy0 = "div by 0";
 
 // dummy routines
-// void   ndNormalize(number& d, const coeffs); // nNormalize...
+void   ndNormalize(number&, const coeffs); // nNormalize...
 
 /// initialize an object of type coeff, return FALSE in case of success
 typedef BOOLEAN (*cfInitCharProc)(coeffs, void *);
 n_coeffType nRegister(n_coeffType n, cfInitCharProc p);
 
+/// initialize an object of type coeffs by its name, return NULL otherwise
+typedef coeffs (*cfInitCfByNameProc)(char *s,n_coeffType n);
+void nRegisterCfByName(cfInitCfByNameProc p,n_coeffType n);
+
 /// find an existing coeff by its "CoeffName"
-coeffs nFindCoeffByName(const char *n);
+coeffs nFindCoeffByName(char *n);
 
 /// divide by the first (leading) number and return it, i.e. make monic
 // void ndClearContent(ICoeffsEnumerator& numberCollectionEnumerator, number& c, const coeffs r);
@@ -104,4 +108,9 @@ coeffs nFindCoeffByName(const char *n);
 /// does nothing (just returns a dummy one number)
 // void ndClearDenominators(ICoeffsEnumerator& numberCollectionEnumerator, number& d, const coeffs r);
 
+/// helper routine: read an int from a string (mod m), return a pointer to the rest
+char* nEati(char *s, int *i, int m);
+
+/// extracts a long integer from s, returns the rest
+char * nEatLong(char *s, mpz_ptr i);
 #endif
