@@ -6,6 +6,7 @@
 */
 #ifdef HAVE_RINGS
 #if SI_INTEGER_VARIANT == 3
+#define POW_2_28 (1L<<60)
 
 //make sure that a small number is an immediate integer
 //bascially copied from longrat.cc nlShort3
@@ -52,11 +53,12 @@ static inline number nrz_short(number x)
 #if CF_DEBUG
   StringAppendS(")");
 #endif
+  nrzTest(x);
   return x;
 }
 
 
-static int nrzSize(number a, const coeffs)
+int nrzSize(number a, const coeffs)
 {
   if (a==INT_TO_SR(0)) return 0;
   if (n_Z_IS_SMALL(a)) return 1;
@@ -92,43 +94,36 @@ number nrzMult (number a, number b, const coeffs R)
   if (n_Z_IS_SMALL(a) && n_Z_IS_SMALL(b))
   {
   //from longrat.cc
-    if (SR_TO_INT(a)==0)
-      return a;
-    if (SR_TO_INT(b)==0)
-      return b;
+    if (a==INT_TO_SR(0)) return INT_TO_SR(0);
+    if (b==INT_TO_SR(0)) return INT_TO_SR(0);
     long r=(long)((unsigned long)(SR_HDL(a)-1L))*((unsigned long)(SR_HDL(b)>>1));
     if ((r/(SR_HDL(b)>>1))==(SR_HDL(a)-1L))
     {
       number u=((number) ((r>>1)+SR_INT));
-    //  if (((((long)SR_HDL(u))<<1)>>1)==SR_HDL(u)) return (u);
+      if (((((long)SR_HDL(u))<<1)>>1)==SR_HDL(u)) return (u);
       return nrzInit(SR_HDL(u)>>2, R);
     }
     mpz_ptr erg = (mpz_ptr) omAllocBin(gmp_nrz_bin);
-    mpz_init(erg);
-    mpz_set_si(erg, SR_TO_INT(a));
+    mpz_init_set_si(erg, SR_TO_INT(a));
     mpz_mul_si(erg, erg, SR_TO_INT(b));
     nrzTest((number)erg);
-    return (number) erg;
+    return nrz_short((number) erg);
   }
   else if (n_Z_IS_SMALL(a))
   {
-    if (SR_TO_INT(a)==0)
-      return a;
+    if (a==INT_TO_SR(0)) return INT_TO_SR(0);
     mpz_ptr erg = (mpz_ptr) omAllocBin(gmp_nrz_bin);
     mpz_init_set(erg, (mpz_ptr) b);
     mpz_mul_si(erg, erg, SR_TO_INT(a));
-    nrzTest((number)erg);
-    return (number) erg;
+    return nrz_short((number) erg);
   }
   else if (n_Z_IS_SMALL(b))
   {
-    if (SR_TO_INT(b)==0)
-      return b;
+    if (b==INT_TO_SR(0)) return INT_TO_SR(0);
     mpz_ptr erg = (mpz_ptr) omAllocBin(gmp_nrz_bin);
     mpz_init_set(erg, (mpz_ptr) a);
     mpz_mul_si(erg, erg, SR_TO_INT(b));
-    nrzTest((number)erg);
-    return (number) erg;
+    return nrz_short((number) erg);
   }
   else
   {
@@ -136,11 +131,10 @@ number nrzMult (number a, number b, const coeffs R)
     mpz_init(erg);
     mpz_mul(erg, (mpz_ptr) a, (mpz_ptr) b);
     nrzTest((number)erg);
-    return (number) erg;
+    return nrz_short((number) erg);
   }
 }
 
-
 static long int_gcd(long a, long b)
 {
   long r;
@@ -154,7 +148,7 @@ static long int_gcd(long a, long b)
     a = b;
     b = r;
   } while (b);
-  return ABS(a); // % in c doeas not imply a signn
+  return ABS(a); // % in C does not imply a sign
                  // it would be unlikely to see a negative here
                  // but who knows
 }
@@ -194,6 +188,7 @@ static number nrzLcm (number a, number b, const coeffs R)
       mpz_lcm(erg, (mpz_ptr) a, (mpz_ptr) b);
     }
   }
+  nrzTest((number)erg);
   return (number) erg;
 }
 
@@ -202,6 +197,7 @@ static number nrzCopy(number a, const coeffs)
   if (n_Z_IS_SMALL(a)) return a;
   mpz_ptr erg = (mpz_ptr) omAllocBin(gmp_nrz_bin);
   mpz_init_set(erg, (mpz_ptr) a);
+  nrzTest((number)erg);
   return (number) erg;
 }
 
@@ -235,6 +231,7 @@ static number nrzGcd (number a,number b,const coeffs R)
     mpz_ptr erg = (mpz_ptr) omAllocBin(gmp_nrz_bin);
     mpz_init(erg);
     mpz_gcd(erg, (mpz_ptr) a, (mpz_ptr) b);
+    nrzTest((number)erg);
     return (number) erg;
   }
 }
@@ -445,51 +442,59 @@ static number _nrzQuotRem (number a, number b, number * r, const coeffs )
 static number nrzQuotRem (number a, number b, number * r, const coeffs )
 #endif
 {
-  assume(SR_TO_INT(b));
-  if (n_Z_IS_SMALL(a) && n_Z_IS_SMALL(b))
-  {
-    if (r)
-      *r = INT_TO_SR(SR_TO_INT(a) % SR_TO_INT(b));
-    return INT_TO_SR(SR_TO_INT(a)/SR_TO_INT(b));
-  }
-  else if (n_Z_IS_SMALL(a))
-  {
-    //a is small, b is not, so q=0, r=a
-    if (r)
-      *r = a;
-    return INT_TO_SR(0);
-  }
-  else if (n_Z_IS_SMALL(b))
-  {
-    unsigned long rr;
-    mpz_ptr qq = (mpz_ptr) omAllocBin(gmp_nrz_bin);
-    mpz_init(qq);
-    mpz_t rrr;
-    mpz_init(rrr);
-    rr = mpz_divmod_ui(qq, rrr, (mpz_ptr) a, (unsigned long)ABS(SR_TO_INT(b)));
-    mpz_clear(rrr);
-
-    if (r)
-      *r = INT_TO_SR(rr);
-    if (SR_TO_INT(b)<0)
-    {
-      mpz_mul_si(qq, qq, -1);
-    }
-    return nrz_short((number)qq);
-  }
-  mpz_ptr qq = (mpz_ptr) omAllocBin(gmp_nrz_bin),
-             rr = (mpz_ptr) omAllocBin(gmp_nrz_bin);
+  mpz_ptr qq = (mpz_ptr) omAllocBin(gmp_nrz_bin);
   mpz_init(qq);
+  mpz_ptr rr = (mpz_ptr) omAllocBin(gmp_nrz_bin);
   mpz_init(rr);
-  mpz_divmod(qq, rr, (mpz_ptr)a, (mpz_ptr)b);
-  if (r)
-    *r = (number) rr;
+  mpz_t bb;
+  mpz_t aa;
+  int gsign;
+  if (SR_HDL(b) & SR_INT)
+  {
+    if (SR_HDL(b)<0) gsign=-1;
+    else             gsign=1;
+    mpz_init_set_si(bb,SR_TO_INT(b));
+  }
   else
   {
-    mpz_clear(rr);
+    gsign = mpz_sgn((mpz_ptr) b);
+    mpz_init_set(bb,(mpz_ptr) b);
   }
-  nrzTest((number)qq);
-  return (number) qq;
+  if (SR_HDL(a) & SR_INT)
+  {
+    mpz_init_set_si(aa,SR_TO_INT(a));
+  }
+  else
+  {
+    mpz_init_set(aa,(mpz_ptr) a);
+  }
+  mpz_t gg, ghalf;
+  mpz_init(gg);
+  mpz_init(ghalf);
+  mpz_abs(gg, bb);
+  mpz_fdiv_qr(qq, rr, aa, gg);
+  mpz_tdiv_q_2exp(ghalf, gg, 1);
+  if (mpz_cmp(rr, ghalf) > 0)  // r > ghalf
+    {
+      mpz_sub(rr, rr, gg);
+      mpz_add_ui(qq, qq, 1);
+    }
+  if (gsign < 0) mpz_neg(qq, qq);
+
+  mpz_clear(ghalf);
+  mpz_clear(gg);
+  mpz_clear(aa);
+  mpz_clear(bb);
+  if (r==NULL)
+  {
+    mpz_clear(rr);
+    omFreeBin(rr,gmp_nrz_bin);
+  }
+  else
+  {
+    *r=nrz_short((number)rr);
+  }
+  return nrz_short((number)qq);
 }
 
 static void nrzPower (number a, int i, number * result, const coeffs)
@@ -524,7 +529,7 @@ static number nrzInitMPZ(mpz_t m, const coeffs)
 }
 
 
-static void nrzDelete(number *a, const coeffs)
+void nrzDelete(number *a, const coeffs)
 {
   if (*a == NULL) return;
   if (n_Z_IS_SMALL(*a)==0)
@@ -566,20 +571,22 @@ static number nrzAdd (number a, number b, const coeffs )
 {
   if (n_Z_IS_SMALL(a) && n_Z_IS_SMALL(b))
   {
-    long c = SR_TO_INT(a) + SR_TO_INT(b);
+    long c = SR_HDL(a)+SR_HDL(b)-1L;
     if (INT_IS_SMALL(c))
-      return INT_TO_SR(c);
+      return (number)c;
     mpz_ptr erg = (mpz_ptr) omAllocBin(gmp_nrz_bin);
-    mpz_init_set_si(erg, c);
-
-    nrzTest((number)erg);
-    return (number) erg;
+    mpz_init_set_si(erg,SR_TO_INT(a));
+    if (SR_HDL(b)>0)
+      mpz_add_ui(erg, erg, (unsigned long)SR_TO_INT(b));
+    else
+      mpz_sub_ui(erg, erg, (unsigned long)-(SR_TO_INT(b)));
+    return nrz_short((number)erg);
   }
   else if (n_Z_IS_SMALL(a))
   {
     mpz_ptr erg = (mpz_ptr) omAllocBin(gmp_nrz_bin);
     mpz_init(erg);
-    if (SR_TO_INT(a)>0)
+    if (SR_HDL(a)>0)
       mpz_add_ui(erg, (mpz_ptr) b, (unsigned long)SR_TO_INT(a));
     else
       mpz_sub_ui(erg, (mpz_ptr) b, (unsigned long)-(SR_TO_INT(a)));
@@ -589,7 +596,7 @@ static number nrzAdd (number a, number b, const coeffs )
   {
     mpz_ptr erg = (mpz_ptr) omAllocBin(gmp_nrz_bin);
     mpz_init(erg);
-    if (SR_TO_INT(b)>0)
+    if (SR_HDL(b)>0)
       mpz_add_ui(erg, (mpz_ptr) a, (unsigned long)SR_TO_INT(b));
     else
       mpz_sub_ui(erg, (mpz_ptr) a, (unsigned long)-(SR_TO_INT(b)));
@@ -608,33 +615,29 @@ static number nrzSub (number a, number b,  const coeffs )
 {
   if (n_Z_IS_SMALL(a) && n_Z_IS_SMALL(b))
   {
-    long c = SR_TO_INT(a) - SR_TO_INT(b);
+    long c = SR_HDL(a)-SR_HDL(b)+1;
     if (INT_IS_SMALL(c))
-      return INT_TO_SR(c);
+      return number(c);
     mpz_ptr erg = (mpz_ptr) omAllocBin(gmp_nrz_bin);
-    mpz_init_set_si(erg, c);
-    nrzTest((number)erg);
-    return (number) erg;
+    mpz_init_set_si(erg,SR_TO_INT(a));
+    if (SR_HDL(b)>0)
+      mpz_sub_ui(erg, erg, (unsigned long)SR_TO_INT(b));
+    else
+      mpz_add_ui(erg, erg, (unsigned long)-(SR_TO_INT(b)));
+    return nrz_short((number)erg);
   }
   else if (n_Z_IS_SMALL(a))
   {
     mpz_ptr erg = (mpz_ptr) omAllocBin(gmp_nrz_bin);
-    mpz_init(erg);
-
-    if (SR_TO_INT(a)>0)
-      mpz_ui_sub(erg, (unsigned long)SR_TO_INT(a), (mpz_ptr) b);
-    else
-    {
-      mpz_add_ui(erg, (mpz_ptr) b, (unsigned long)-SR_TO_INT(a));
-      mpz_neg(erg, erg);
-    }
+    mpz_init_set_si(erg,SR_TO_INT(a));
+    mpz_sub(erg, erg, (mpz_ptr) b);
     return nrz_short((number) erg);
   }
   else if (n_Z_IS_SMALL(b))
   {
     mpz_ptr erg = (mpz_ptr) omAllocBin(gmp_nrz_bin);
     mpz_init(erg);
-    if (SR_TO_INT(b)>0)
+    if (SR_HDL(b)>0)
       mpz_sub_ui(erg, (mpz_ptr) a, (unsigned long)SR_TO_INT(b));
     else
       mpz_add_ui(erg, (mpz_ptr) a, (unsigned long)-SR_TO_INT(b));
@@ -740,22 +743,28 @@ static int nrzDivComp(number a, number b, const coeffs r)
   return 0;
 }
 
-static number nrzDiv (number a,number b, const coeffs)
+static number nrzDiv (number a,number b, const coeffs cf)
 {
-  assume(SR_TO_INT(b));
-  if (nrzIsZero(b))
+  if (nrzIsZero(b,cf))
   {
     WerrorS(nDivBy0);
     return INT_TO_SR(0);
   }
   else if (n_Z_IS_SMALL(a) && n_Z_IS_SMALL(b))
   {
+    long i=SR_TO_INT(a);
+    long j=SR_TO_INT(b);
+    if (j==1L) return a;
+    if ((i==-POW_2_28) && (j== -1L))
+    {
+      return nrzInit(POW_2_28,cf);
+    }
     //if (SR_TO_INT(a) % SR_TO_INT(b))
     //{
     //  WerrorS("1:Division by non divisible element.");
     //  WerrorS("Result is without remainder.");
     //}
-    return INT_TO_SR(SR_TO_INT(a)/SR_TO_INT(b));
+    return INT_TO_SR(i/j);
   }
   else if (n_Z_IS_SMALL(a))
   {
@@ -777,7 +786,7 @@ static number nrzDiv (number a,number b, const coeffs)
     //  WerrorS("Result is without remainder.");
     }
     mpz_clear(r);
-    if (SR_TO_INT(b)<0)
+    if (SR_HDL(b)<0)
       mpz_neg(erg, erg);
     return nrz_short((number) erg);
   }
@@ -810,11 +819,11 @@ static number nrzDiv (number a,number b, const coeffs)
   return nrz_short((number) erg);
 }
 
-static number nrzExactDiv (number a,number b, const coeffs)
+static number nrzExactDiv (number a,number b, const coeffs cf)
 {
   assume(SR_TO_INT(b));
   mpz_t aa, bb;
-  if (nrzIsZero(b))
+  if (nrzIsZero(b,cf))
   {
     WerrorS(nDivBy0);
     return INT_TO_SR(0);
@@ -832,8 +841,7 @@ static number nrzExactDiv (number a,number b, const coeffs)
   mpz_tdiv_q(erg, (mpz_ptr) aa, (mpz_ptr) bb);
   mpz_clear(aa);
   mpz_clear(bb);
-  nrzTest((number)erg);
-  return (number) erg;
+  return nrz_short((number) erg);
 }
 
 static number nrzIntMod (number a,number b, const coeffs)
@@ -948,6 +956,7 @@ static number nrzFarey(number r, number N, const coeffs R)
 
   nrzDelete(&a1, R);
   nrzDelete(&b1, R);
+  nrzTest(ab);
   return ab;
 }
 
@@ -1012,18 +1021,6 @@ static nMapFunc nrzSetMap(const coeffs src, const coeffs /*dst*/)
 }
 
 
-/*
- * set the exponent (allocate and init tables) (TODO)
- */
-
-void nrzSetExp(int, coeffs)
-{
-}
-
-void nrzInitExp(int, coeffs)
-{
-}
-
 #ifdef LDEBUG
 BOOLEAN nrzDBTest (number x, const char *f, const int l, const coeffs)
 {
@@ -1154,6 +1151,23 @@ static void nrzMPZ(mpz_t res, number &a, const coeffs)
     mpz_init_set(res, (mpz_ptr) a);
 }
 
+static number nrzEucNorm (number a, const coeffs )
+{
+  if (n_Z_IS_SMALL(a))
+  {
+    long aa=ABS(SR_TO_INT(a));
+    return INT_TO_SR(aa);
+  }
+  else
+  {
+    mpz_ptr abs = (mpz_ptr) omAllocBin(gmp_nrz_bin);
+    mpz_init(abs);
+    mpz_abs(abs, (mpz_ptr)a);
+    nrzTest((number)abs);
+    return (number) abs;
+  }
+}
+
 static coeffs nrzQuot1(number c, const coeffs r)
 {
     mpz_t dummy;
@@ -1206,7 +1220,7 @@ BOOLEAN nrzInitChar(coeffs r,  void *)
   r->cfExtGcd = nrzExtGcd; // only for ring stuff
   r->cfXExtGcd = nrzXExtGcd; // only for ring stuff
   r->cfEucNorm = nrzEucNorm;
-  r->cfQuotRem = nrzSmallestQuotRem;
+  r->cfQuotRem = nrzQuotRem;
   r->cfDivBy = nrzDivBy; // only for ring stuff
   //#endif
   r->cfInpNeg   = nrzNeg;
