@@ -42,12 +42,24 @@ static inline number nr2mMultM(number a, number b, const coeffs r)
     ((((unsigned long) a) * ((unsigned long) b)) & r->mod2mMask);
 }
 
+static inline void nr2mInpMultM(number &a, number b, const coeffs r)
+{
+  a= (number)
+    ((((unsigned long) a) * ((unsigned long) b)) & r->mod2mMask);
+}
+
 static inline number nr2mAddM(number a, number b, const coeffs r)
 {
   return (number)
     ((((unsigned long) a) + ((unsigned long) b)) & r->mod2mMask);
 }
 
+static inline void nr2mInpAddM(number &a, number b, const coeffs r)
+{
+  a= (number)
+    ((((unsigned long) a) + ((unsigned long) b)) & r->mod2mMask);
+}
+
 static inline number nr2mSubM(number a, number b, const coeffs r)
 {
   return (number)((unsigned long)a < (unsigned long)b ?
@@ -141,6 +153,15 @@ static number nr2mMult(number a, number b, const coeffs r)
   return n;
 }
 
+static void nr2mInpMult(number &a, number b, const coeffs r)
+{
+  if (((unsigned long)a == 0) || ((unsigned long)b == 0))
+  { a=(number)0;  return; }
+  else
+    nr2mInpMultM(a, b, r);
+  n_Test(a,r);
+}
+
 static number nr2mAnn(number b, const coeffs r);
 /*
  * Give the smallest k, such that a * x = k = b * y has a solution
@@ -238,16 +259,16 @@ static void specialXGCD(unsigned long& s, unsigned long a, const coeffs r)
   }
   s = mpz_get_ui(u1); /* now: 0 <= s <= 2^m - 1 */
 
-  mpz_clear(u);  omFree((ADDRESS)u);
-  mpz_clear(u0); omFree((ADDRESS)u0);
-  mpz_clear(u1); omFree((ADDRESS)u1);
-  mpz_clear(u2); omFree((ADDRESS)u2);
-  mpz_clear(v);  omFree((ADDRESS)v);
-  mpz_clear(v0); omFree((ADDRESS)v0);
-  mpz_clear(v1); omFree((ADDRESS)v1);
-  mpz_clear(v2); omFree((ADDRESS)v2);
-  mpz_clear(q); omFree((ADDRESS)q);
-  mpz_clear(rr); omFree((ADDRESS)rr);
+  mpz_clear(u);  omFreeBinAddr((ADDRESS)u);
+  mpz_clear(u0); omFreeBinAddr((ADDRESS)u0);
+  mpz_clear(u1); omFreeBinAddr((ADDRESS)u1);
+  mpz_clear(u2); omFreeBinAddr((ADDRESS)u2);
+  mpz_clear(v);  omFreeBinAddr((ADDRESS)v);
+  mpz_clear(v0); omFreeBinAddr((ADDRESS)v0);
+  mpz_clear(v1); omFreeBinAddr((ADDRESS)v1);
+  mpz_clear(v2); omFreeBinAddr((ADDRESS)v2);
+  mpz_clear(q); omFreeBinAddr((ADDRESS)q);
+  mpz_clear(rr); omFreeBinAddr((ADDRESS)rr);
 }
 
 static unsigned long InvMod(unsigned long a, const coeffs r)
@@ -327,7 +348,7 @@ static void nr2mPower(number a, int i, number * result, const coeffs r)
  */
 static number nr2mInit(long i, const coeffs r)
 {
-  if (i == 0) return (number)(unsigned long)i;
+  if (i == 0) return (number)(unsigned long)0;
 
   long ii = i;
   unsigned long j = (unsigned long)1;
@@ -335,7 +356,7 @@ static number nr2mInit(long i, const coeffs r)
   unsigned long k = (unsigned long)ii;
   k = k & r->mod2mMask;
   /* now we have: i = j * k mod 2^m */
-  return (number)nr2mMult((number)j, (number)k, r);
+  return nr2mMult((number)j, (number)k, r);
 }
 
 /*
@@ -359,6 +380,12 @@ static number nr2mAdd(number a, number b, const coeffs r)
   return n;
 }
 
+static void nr2mInpAdd(number &a, number b, const coeffs r)
+{
+  nr2mInpAddM(a, b, r);
+  n_Test(a,r);
+}
+
 static number nr2mSub(number a, number b, const coeffs r)
 {
   number n=nr2mSubM(a, b, r);
@@ -424,7 +451,7 @@ static number nr2mDiv(number a, number b, const coeffs r)
       return (number) ((unsigned long) a / (unsigned long) b);
     }
   }
-  number n=(number)nr2mMult(a, nr2mInversM(b,r),r);
+  number n=nr2mMult(a, nr2mInversM(b,r),r);
   n_Test(n,r);
   return n;
 }
@@ -585,7 +612,7 @@ static number nr2mAnn(number b, const coeffs r)
     mpz_init_set_ui(cc, r->mod2mMask); mpz_add_ui(cc, cc, 1);
     mpz_div_ui(cc, cc, (unsigned long)(unsigned long)b);
     unsigned long s = mpz_get_ui(cc);
-    mpz_clear(cc); omFree((ADDRESS)cc);
+    mpz_clear(cc); omFreeBinAddr((ADDRESS)cc);
     return (number)(unsigned long)s;
   }
 }
@@ -618,7 +645,7 @@ number nr2mMapZp(number from, const coeffs /*src*/, const coeffs dst)
   unsigned long i = (unsigned long)ii;
   i = i & dst->mod2mMask;
   /* now we have: from = j * i mod 2^m */
-  return (number)nr2mMult((number)i, (number)j, dst);
+  return nr2mMult((number)i, (number)j, dst);
 }
 
 static number nr2mMapGMP(number from, const coeffs /*src*/, const coeffs dst)
@@ -631,8 +658,8 @@ static number nr2mMapGMP(number from, const coeffs /*src*/, const coeffs dst)
   mpz_and(erg, (mpz_ptr)from, k);
   number res = (number) mpz_get_ui(erg);
 
-  mpz_clear(erg); omFree((ADDRESS)erg);
-  mpz_clear(k);   omFree((ADDRESS)k);
+  mpz_clear(erg); omFreeBinAddr((ADDRESS)erg);
+  mpz_clear(k);   omFreeBinAddr((ADDRESS)k);
 
   return (number)res;
 }
@@ -642,7 +669,7 @@ static number nr2mMapQ(number from, const coeffs src, const coeffs dst)
   mpz_ptr gmp = (mpz_ptr)omAllocBin(gmp_nrz_bin);
   nlMPZ(gmp, from, src);
   number res=nr2mMapGMP((number)gmp,src,dst);
-  mpz_clear(gmp); omFree((ADDRESS)gmp);
+  mpz_clear(gmp); omFreeBinAddr((ADDRESS)gmp);
   return res;
 }
 
@@ -792,8 +819,10 @@ BOOLEAN nr2mInitChar (coeffs r, void* p)
   //r->cfCopy        = ndCopy;
   r->cfInt         = nr2mInt;
   r->cfAdd         = nr2mAdd;
+  r->cfInpAdd      = nr2mInpAdd;
   r->cfSub         = nr2mSub;
   r->cfMult        = nr2mMult;
+  r->cfInpMult     = nr2mInpMult;
   r->cfDiv         = nr2mDiv;
   r->cfAnn         = nr2mAnn;
   r->cfIntMod      = nr2mMod;
