@@ -27,7 +27,7 @@
 
 void nrnWrite (number a, const coeffs);
 #ifdef LDEBUG
-BOOLEAN nrnDBTest      (number a, const char *f, const int l, const coeffs r);
+static BOOLEAN nrnDBTest      (number a, const char *f, const int l, const coeffs r);
 #endif
 
 EXTERN_VAR omBin gmp_nrz_bin;
@@ -188,10 +188,7 @@ static void nrnDelete(number *a, const coeffs)
 }
 static int nrnSize(number a, const coeffs)
 {
-  mpz_ptr p=(mpz_ptr)a;
-  int s=p->_mp_alloc;
-  if (s==1) s=(mpz_cmp_ui(p,0)!=0);
-  return s;
+  return mpz_size1((mpz_ptr)a);
 }
 #endif
 /*
@@ -206,6 +203,12 @@ static number nrnMult(number a, number b, const coeffs r)
   return (number) erg;
 }
 
+static void nrnInpMult(number &a, number b, const coeffs r)
+{
+  mpz_mul((mpz_ptr)a, (mpz_ptr)a, (mpz_ptr) b);
+  mpz_mod((mpz_ptr)a, (mpz_ptr)a, r->modNumber);
+}
+
 static void nrnPower(number a, int i, number * result, const coeffs r)
 {
   mpz_ptr erg = (mpz_ptr)omAllocBin(gmp_nrz_bin);
@@ -223,6 +226,12 @@ static number nrnAdd(number a, number b, const coeffs r)
   return (number) erg;
 }
 
+static void nrnInpAdd(number &a, number b, const coeffs r)
+{
+  mpz_add((mpz_ptr)a, (mpz_ptr)a, (mpz_ptr) b);
+  mpz_mod((mpz_ptr)a, (mpz_ptr)a, r->modNumber);
+}
+
 static number nrnSub(number a, number b, const coeffs r)
 {
   mpz_ptr erg = (mpz_ptr)omAllocBin(gmp_nrz_bin);
@@ -234,7 +243,7 @@ static number nrnSub(number a, number b, const coeffs r)
 
 static BOOLEAN nrnIsZero(number a, const coeffs)
 {
-  return 0 == mpz_cmpabs_ui((mpz_ptr)a, 0);
+  return 0 == mpz_sgn1((mpz_ptr)a);
 }
 
 static number nrnNeg(number c, const coeffs r)
@@ -905,7 +914,7 @@ static void nrnInitExp(unsigned long m, coeffs r)
 }
 
 #ifdef LDEBUG
-BOOLEAN nrnDBTest (number a, const char *f, const int l, const coeffs r)
+static BOOLEAN nrnDBTest (number a, const char *f, const int l, const coeffs r)
 {
   if ( (mpz_sgn1((mpz_ptr) a) < 0) || (mpz_cmp((mpz_ptr) a, r->modNumber) > 0) )
   {
@@ -1005,8 +1014,10 @@ BOOLEAN nrnInitChar (coeffs r, void* p)
   r->cfSize        = nrnSize;
   r->cfInt         = nrnInt;
   r->cfAdd         = nrnAdd;
+  r->cfInpAdd      = nrnInpAdd;
   r->cfSub         = nrnSub;
   r->cfMult        = nrnMult;
+  r->cfInpMult     = nrnInpMult;
   r->cfDiv         = nrnDiv;
   r->cfAnn         = nrnAnn;
   r->cfIntMod      = nrnMod;
