@@ -7,15 +7,17 @@
 * ABSTRACT: class intvec: lists/vectors of integers
 */
 #include <string.h>
+#include "misc/auxiliary.h"
 #include "omalloc/omalloc.h"
-#ifndef XMEMORY_H
+#ifdef HAVE_OMALLOC
 #include "omalloc/omallocClass.h"
 #endif
+
 #include "reporter/reporter.h"
 
 
 class intvec
-#ifndef XMEMORY_H
+#ifdef HAVE_OMALLOC
              :public omallocClass
 #endif
 {
@@ -126,9 +128,19 @@ public:
     }
     return m;
   }
+  inline int max_in()
+  {
+    int m=0;
+    if (row>0)
+    {
+      m=v[0];
+      for (int i=row*col-1; i>0; i--) if (v[i]>m) m=v[i];
+    }
+    return m;
+  }
   intvec* delete_pos(int p);
   // keiner (ausser obachman) darf das folgenden benutzen !!!
-  inline int * ivGetVec() { return v; }
+  inline int * ivGetVec() const { return v; }
 };
 inline intvec * ivCopy(const intvec * o)
 {
@@ -138,6 +150,7 @@ inline intvec * ivCopy(const intvec * o)
 }
 
 intvec * ivAdd(intvec * a, intvec * b);
+intvec * ivAddShift(intvec * a, intvec * b, int s);
 intvec * ivSub(intvec * a, intvec * b);
 intvec * ivTranp(intvec * o);
 int      ivTrace(intvec * o);
