@@ -22,6 +22,8 @@
 #include "polys/templates/p_MemCmp.h"
 #include "polys/templates/p_MemAdd.h"
 #include "polys/templates/p_MemCopy.h"
+#include "polys/flintconv.h"
+#include "polys/flint_mpoly.h"
 
 #include "p_Mult_q.h"
 
@@ -64,6 +66,36 @@ BOOLEAN pqLength(poly p, poly q, int &lp, int &lq, const int min)
   while (1);
 }
 
+static void pqLengthApprox(poly p, poly q, int &lp, int &lq, const int min)
+{
+  int l = 0;
+
+  do
+  {
+    if (p == NULL)
+    {
+      lp=l;
+      lq=l+(q!=NULL);
+      return;
+    }
+    if (q == NULL) /* && p!=NULL */
+    {
+      lp=l+1;
+      lq=l;
+      return;
+    }
+    if (l>min) /* && p,q!=NULL */
+    {
+      lp=l; lq=l;
+      return;
+    }
+    pIter(p);
+    pIter(q);
+    l++;
+  }
+  while (1);
+}
+
 
 static poly _p_Mult_q_Bucket(poly p, const int lp,
                              poly q, const int lq,
@@ -267,9 +299,17 @@ static poly _p_Mult_q_Normal(poly p, poly q, const int copy, const ring r)
 }
 
 
+// Use factory if min(pLength(p), pLength(q)) >= MIN_LENGTH_FACTORY (>MIN_LENGTH_BUCKET)
+// Not thoroughly tested what is best
+#define MIN_LENGTH_FACTORY 200
+#define MIN_LENGTH_FACTORY_QQ 60
+#define MIN_FLINT_QQ 10
+#define MIN_FLINT_Zp 20
+#define MIN_FLINT_Z 10
+
 /// Returns:  p * q,
 /// Destroys: if !copy then p, q
-/// Assumes: pLength(p) >= 2 pLength(q) >=2
+/// Assumes: pLength(p) >= 2 pLength(q) >=2, !rIsPluralRing(r)
 poly _p_Mult_q(poly p, poly q, const int copy, const ring r)
 {
   assume(r != NULL);
@@ -280,7 +320,8 @@ poly _p_Mult_q(poly p, poly q, const int copy, const ring r)
   int lp, lq, l;
   poly pt;
 
-  pqLength(p, q, lp, lq, MIN_LENGTH_BUCKET);
+  // MIN_LENGTH_FACTORY must be >= MIN_LENGTH_FACTORY_QQ, MIN_FLINT_QQ, MIN_FLINT_Zp 20
+  pqLengthApprox(p, q, lp, lq, MIN_LENGTH_FACTORY);
 
   if (lp < lq)
   {
@@ -291,10 +332,65 @@ poly _p_Mult_q(poly p, poly q, const int copy, const ring r)
     lp = lq;
     lq = l;
   }
+  BOOLEAN pure_polys=(p_GetComp(p,r)==0) && (p_GetComp(q,r)==0);
+  #ifdef HAVE_FLINT
+  #if __FLINT_RELEASE >= 20503
+  if (lq>MIN_FLINT_QQ)
+  {
+    fmpq_mpoly_ctx_t ctx;
+    if (pure_polys && rField_is_Q(r) && !convSingRFlintR(ctx,r))
+    {
+      // lq is a lower bound for the length of p and  q
+      poly res=Flint_Mult_MP(p,lq,q,lq,ctx,r);
+      if (!copy)
+      {
+        p_Delete(&p,r);
+        p_Delete(&q,r);
+      }
+      return res;
+    }
+  }
+  if (lq>MIN_FLINT_Zp)
+  {
+    nmod_mpoly_ctx_t ctx;
+    if (pure_polys && rField_is_Zp(r) && !convSingRFlintR(ctx,r))
+    {
+      // lq is a lower bound for the length of p and  q
+      poly res=Flint_Mult_MP(p,lq,q,lq,ctx,r);
+      if (!copy)
+      {
+        p_Delete(&p,r);
+        p_Delete(&q,r);
+      }
+      return res;
+    }
+  }
+  if (lq>MIN_FLINT_Z)
+  {
+    fmpz_mpoly_ctx_t ctx;
+    if (pure_polys && rField_is_Z(r) && !convSingRFlintR(ctx,r))
+    {
+      // lq is a lower bound for the length of p and  q
+      poly res=Flint_Mult_MP(p,lq,q,lq,ctx,r);
+      if (!copy)
+      {
+        p_Delete(&p,r);
+        p_Delete(&q,r);
+      }
+      return res;
+    }
+  }
+  #endif
+  #endif
   if (lq < MIN_LENGTH_BUCKET || TEST_OPT_NOT_BUCKETS)
     return _p_Mult_q_Normal(p, q, copy, r);
-  else if ((lq >= MIN_LENGTH_FACTORY)
-  && (r->cf->convSingNFactoryN!=ndConvSingNFactoryN))
+  else if (pure_polys
+  && ((r->cf->extRing==NULL)||(r->cf->extRing->qideal!=NULL))
+    /* exclude trans. extensions: may contain rat.funct as cf */
+  && (((lq >= MIN_LENGTH_FACTORY)
+      && (r->cf->convSingNFactoryN!=ndConvSingNFactoryN))
+    || ((lq >= MIN_LENGTH_FACTORY_QQ)
+      && rField_is_Q(r))))
   {
     poly h=singclap_pmult(p,q,r);
     if (!copy)
@@ -306,8 +402,8 @@ poly _p_Mult_q(poly p, poly q, const int copy, const ring r)
   }
   else
   {
-    assume(lp == pLength(p));
-    assume(lq == pLength(q));
+    lp=pLength(p);
+    lq=pLength(q);
     return _p_Mult_q_Bucket(p, lp, q, lq, copy, r);
   }
 }
