@@ -184,9 +184,10 @@ void idSkipZeroes (ideal ide)
 
   int k;
   int j = -1;
+  int idelems=IDELEMS(ide);
   BOOLEAN change=FALSE;
 
-  for (k=0; k<IDELEMS(ide); k++)
+  for (k=0; k<idelems; k++)
   {
     if (ide->m[k] != NULL)
     {
@@ -207,28 +208,15 @@ void idSkipZeroes (ideal ide)
       j = 0;
     else
     {
-      for (k=j+1; k<IDELEMS(ide); k++)
+      for (k=j+1; k<idelems; k++)
         ide->m[k] = NULL;
     }
-    pEnlargeSet(&(ide->m),IDELEMS(ide),j+1-IDELEMS(ide));
-    IDELEMS(ide) = j+1;
+    j++;
+    pEnlargeSet(&(ide->m),idelems,j-idelems);
+    IDELEMS(ide) = j;
   }
 }
 
-/// count non-zero elements
-int idElem(const ideal F)
-{
-  assume (F != NULL);
-
-  int i=0;
-
-  for(int j=IDELEMS(F)-1;j>=0;j--)
-  {
-    if ((F->m)[j]!=NULL) i++;
-  }
-  return i;
-}
-
 /// copies the first k (>= 1) entries of the given ideal/module
 /// and returns these as a new ideal/module
 /// (Note that the copied entries may be zero.)
@@ -346,6 +334,87 @@ void id_DelLmEquals(ideal id, const ring r)
   }
 }
 
+/// delete id[j], if LT(j) == coeff*mon*LT(i)
+static void id_DelDiv_SEV(ideal id, int k,const ring r)
+{
+  int kk = k+1;
+  long *sev=(long*)omAlloc0(kk*sizeof(long));
+  while(id->m[k]==NULL) k--;
+  BOOLEAN only_lm=r->cf->has_simple_Alloc;
+  if (only_lm)
+  {
+    for (int i=k; i>=0; i--)
+    {
+      if((id->m[i]!=NULL) && (pNext(id->m[i])!=NULL))
+      {
+        only_lm=FALSE;
+        break;
+      }
+    }
+  }
+  for (int i=k; i>=0; i--)
+  {
+    if(id->m[i]!=NULL)
+    {
+      sev[i]=p_GetShortExpVector(id->m[i],r);
+    }
+  }
+  if (only_lm)
+  {
+    for (int i=0; i<k; i++)
+    {
+      if (id->m[i] != NULL)
+      {
+        poly m_i=id->m[i];
+        long sev_i=sev[i];
+        for (int j=i+1; j<=k; j++)
+        {
+          if (id->m[j]!=NULL)
+          {
+            if (p_LmShortDivisibleBy(m_i, sev_i,id->m[j],~sev[j],r))
+            {
+              p_LmFree(&id->m[j],r);
+            }
+            else if (p_LmShortDivisibleBy(id->m[j],sev[j], m_i,~sev_i,r))
+            {
+              p_LmFree(&id->m[i],r);
+              break;
+            }
+          }
+        }
+      }
+    }
+  }
+  else
+  {
+    for (int i=0; i<k; i++)
+    {
+      if (id->m[i] != NULL)
+      {
+        poly m_i=id->m[i];
+        long sev_i=sev[i];
+        for (int j=i+1; j<=k; j++)
+        {
+          if (id->m[j]!=NULL)
+          {
+            if (p_LmShortDivisibleBy(m_i, sev_i, id->m[j],~sev[j],r))
+            {
+              p_Delete(&id->m[j],r);
+            }
+            else if (p_LmShortDivisibleBy(id->m[j],sev[j], m_i,~sev_i,r))
+            {
+              p_Delete(&id->m[i],r);
+              break;
+            }
+          }
+        }
+      }
+    }
+  }
+  omFreeSize(sev,kk*sizeof(long));
+}
+
+
 /// delete id[j], if LT(j) == coeff*mon*LT(i) and vice versa, i.e.,
 /// delete id[i], if LT(i) == coeff*mon*LT(j)
 void id_DelDiv(ideal id, const ring r)
@@ -354,16 +423,16 @@ void id_DelDiv(ideal id, const ring r)
 
   int i, j;
   int k = IDELEMS(id)-1;
-  for (i=k; i>=0; i--)
-  {
-    if (id->m[i] != NULL)
-    {
-      for (j=k; j>i; j--)
-      {
-        if (id->m[j]!=NULL)
-        {
 #ifdef HAVE_RINGS
-          if (rField_is_Ring(r))
+  if (rField_is_Ring(r))
+  {
+    for (i=k-1; i>=0; i--)
+    {
+      if (id->m[i] != NULL)
+      {
+        for (j=k; j>i; j--)
+        {
+          if (id->m[j]!=NULL)
           {
             if (p_DivisibleByRingCase(id->m[i], id->m[j],r))
             {
@@ -375,15 +444,32 @@ void id_DelDiv(ideal id, const ring r)
               break;
             }
           }
-          else
+        }
+      }
+    }
+  }
+  else
 #endif
+  {
+    /* the case of a coefficient field: */
+    if (k>9)
+    {
+      id_DelDiv_SEV(id,k,r);
+      return;
+    }
+    for (i=k-1; i>=0; i--)
+    {
+      if (id->m[i] != NULL)
+      {
+        for (j=k; j>i; j--)
+        {
+          if (id->m[j]!=NULL)
           {
-            /* the case of a coefficient field: */
-            if (p_DivisibleBy(id->m[i], id->m[j],r))
+            if (p_LmDivisibleBy(id->m[i], id->m[j],r))
             {
               p_Delete(&id->m[j],r);
             }
-            else if (p_DivisibleBy(id->m[j], id->m[i],r))
+            else if (p_LmDivisibleBy(id->m[j], id->m[i],r))
             {
               p_Delete(&id->m[i],r);
               break;
@@ -471,6 +557,60 @@ void id_DBTest(ideal h1, int level, const char *f,const int l, const ring r, con
 }
 #endif
 
+#ifdef PDEBUG
+/// Internal verification for ideals/modules and dense matrices!
+void id_DBLmTest(ideal h1, int level, const char *f,const int l, const ring r)
+{
+  if (h1 != NULL)
+  {
+    // assume(IDELEMS(h1) > 0); for ideal/module, does not apply to matrix
+    omCheckAddrSize(h1,sizeof(*h1));
+
+    assume( h1->ncols >= 0 );
+    assume( h1->nrows >= 0 ); // matrix case!
+
+    assume( h1->rank >= 0 );
+
+    const long n = ((long)h1->ncols * (long)h1->nrows);
+
+    assume( !( n > 0 && h1->m == NULL) );
+
+    if( h1->m != NULL && n > 0 )
+      omdebugAddrSize(h1->m, n * sizeof(poly));
+
+    long new_rk = 0; // inlining id_RankFreeModule(h1, r, tailRing);
+
+    /* to be able to test matrices: */
+    for (long i=n - 1; i >= 0; i--)
+    {
+      if (h1->m[i]!=NULL)
+      {
+        _p_LmTest(h1->m[i], r, level);
+        const long k = p_GetComp(h1->m[i], r);
+        if (k > new_rk) new_rk = k;
+      }
+    }
+
+    // dense matrices only contain polynomials:
+    // h1->nrows == h1->rank > 1 && new_rk == 0!
+    assume( !( h1->nrows == h1->rank && h1->nrows > 1 && new_rk > 0 ) ); //
+
+    if(new_rk > h1->rank)
+    {
+      dReportError("wrong rank %d (should be %d) in %s:%d\n",
+                   h1->rank, new_rk, f,l);
+      omPrintAddrInfo(stderr, h1, " for ideal");
+      h1->rank = new_rk;
+    }
+  }
+  else
+  {
+    Print("error: ideal==NULL in %s:%d\n",f,l);
+    assume( h1 != NULL );
+  }
+}
+#endif
+
 /// for idSort: compare a and b revlex inclusive module comp.
 static int p_Comp_RevLex(poly a, poly b,BOOLEAN nolex, const ring R)
 {
@@ -790,8 +930,6 @@ BOOLEAN idIs0 (ideal h)
 /// return the maximal component number found in any polynomial in s
 long id_RankFreeModule (ideal s, ring lmRing, ring tailRing)
 {
-  id_TestTail(s, lmRing, tailRing);
-
   long j = 0;
 
   if (rRing_has_Comp(tailRing) && rRing_has_Comp(lmRing))
@@ -835,6 +973,52 @@ BOOLEAN id_HomIdeal (ideal id, ideal Q, const ring r)
   return b;
 }
 
+BOOLEAN id_HomIdealW (ideal id, ideal Q,  const intvec *w, const ring r)
+{
+  int i;
+  BOOLEAN b;
+  i = 0;
+  b = TRUE;
+  while ((i < IDELEMS(id)) && b)
+  {
+    b = p_IsHomogeneousW(id->m[i],w,r);
+    i++;
+  }
+  if ((b) && (Q!=NULL) && (IDELEMS(Q)>0))
+  {
+    i=0;
+    while ((i < IDELEMS(Q)) && b)
+    {
+      b = p_IsHomogeneousW(Q->m[i],w,r);
+      i++;
+    }
+  }
+  return b;
+}
+
+BOOLEAN id_HomModuleW (ideal id, ideal Q,  const intvec *w, const intvec *module_w, const ring r)
+{
+  int i;
+  BOOLEAN b;
+  i = 0;
+  b = TRUE;
+  while ((i < IDELEMS(id)) && b)
+  {
+    b = p_IsHomogeneousW(id->m[i],w,module_w,r);
+    i++;
+  }
+  if ((b) && (Q!=NULL) && (IDELEMS(Q)>0))
+  {
+    i=0;
+    while ((i < IDELEMS(Q)) && b)
+    {
+      b = p_IsHomogeneousW(Q->m[i],w,r);
+      i++;
+    }
+  }
+  return b;
+}
+
 /*2
 *initialized a field with r numbers between beg and end for the
 *procedure idNextChoise
@@ -1194,9 +1378,18 @@ ideal id_Head(ideal h,const ring r)
 {
   ideal m = idInit(IDELEMS(h),h->rank);
 
-  for (int i=IDELEMS(h)-1;i>=0; i--)
-    if (h->m[i]!=NULL)
-      m->m[i]=p_Head(h->m[i],r);
+  if (r->cf->has_simple_Alloc)
+  {
+    for (int i=IDELEMS(h)-1;i>=0; i--)
+      if (h->m[i]!=NULL)
+        m->m[i]=p_CopyPowerProduct0(h->m[i],pGetCoeff(h->m[i]),r);
+  }
+  else
+  {
+    for (int i=IDELEMS(h)-1;i>=0; i--)
+      if (h->m[i]!=NULL)
+        m->m[i]=p_Head(h->m[i],r);
+  }
 
   return m;
 }
@@ -1217,7 +1410,7 @@ ideal id_Homogen(ideal h, int varnum,const ring r)
 ideal id_Vec2Ideal(poly vec, const ring R)
 {
    ideal result=idInit(1,1);
-   omFree((ADDRESS)result->m);
+   omFreeBinAddr((ADDRESS)result->m);
    p_Vec2Polys(vec, &(result->m), &(IDELEMS(result)),R);
    return result;
 }
