@@ -47,10 +47,10 @@ const BOOLEAN p_procs_dynamic = TRUE;
 // define to bound for length of p_Proc name
 #define MAX_PROCNAME_LEN 200
 
-static void* p_procs_handle_FieldIndep = NULL;
-static void* p_procs_handle_FieldZp = NULL;
-static void* p_procs_handle_FieldQ = NULL;
-static void* p_procs_handle_FieldGeneral = NULL;
+void* p_procs_handle_FieldIndep = NULL;
+void* p_procs_handle_FieldZp = NULL;
+void* p_procs_handle_FieldQ = NULL;
+void* p_procs_handle_FieldGeneral = NULL;
 
 static void* p_ProcInitHandle(void** handle, const char* module)
 {
@@ -144,8 +144,6 @@ static void* GetGeneralProc(p_Proc proc)
 }
 
 #ifdef RDEBUG
-#include "omalloc/omalloc.h"
-
 static const char* GetGeneralProcName(p_Proc proc)
 {
   switch(proc)
