@@ -21,7 +21,7 @@
 /***************************************************************
  Here is how it works:
  At run-time, SetProcs is used to choose the appropriate PolyProcs
-              based on the ring properies.
+              based on the ring properties.
  At generate-time, SetProcs is used to generate all
               possible PolyProcs.
  Which PolyProcs are generated/used can be controled by values of
@@ -634,7 +634,7 @@ static inline int index(p_Proc proc, p_Field field, p_Length length, p_Ord ord)
         return index(length, ord);
 
       default:
-        assume(0);
+        printf("error in index (%d)- should not happen", (int)proc);
         return -1;
   }
 }
